""" Unit tests for `core_linked_records_app.utils.data`.
"""
from copy import deepcopy
from unittest import TestCase
from unittest.mock import MagicMock, patch

from core_main_app.components.template.models import Template
from xml_utils.commons import exceptions as xml_utils_exceptions
from core_linked_records_app.utils import data as data_utils, exceptions


class TestSetPidValueForDataForXMLData(TestCase):
    """Unit tests for `set_pid_value_for_data` function, limiting to XML data."""

    def setUp(self):
        """setUp"""
        template_format = Template.XSD
        self.mock_data = MagicMock()
        self.mock_data.template.format = template_format

        self.kwargs = {
            "data": self.mock_data,
            "pid_path": "mock_pid_path",
            "pid_value": "pid_value",
        }

    @patch.object(data_utils, "create_tree_from_xpath")
    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_get_xpath_with_target_namespace_called(
        self, mock_pid_xml_utils, mock_xsd_tree, mock_create_tree_from_xpath
    ):
        """test_get_xpath_with_target_namespace_called"""
        data_utils.set_pid_value_for_data(**self.kwargs)

        mock_pid_xml_utils.get_xpath_with_target_namespace.assert_called_with(
            mock_pid_xml_utils.get_xpath_from_dot_notation(
                self.kwargs["pid_path"]
            ),
            self.kwargs["data"].template.content,
        )

    @patch.object(data_utils, "create_tree_from_xpath")
    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_get_target_namespace_for_xsd_string_called(
        self, mock_pid_xml_utils, mock_xsd_tree, mock_create_tree_from_xpath
    ):
        """test_get_target_namespace_for_xsd_string_called"""
        data_utils.set_pid_value_for_data(**self.kwargs)

        mock_pid_xml_utils.get_target_namespace_for_xsd_string.assert_called_with(
            self.kwargs["data"].template.content
        )

    @patch.object(data_utils, "create_tree_from_xpath")
    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_xsdtree_build_tree_called(
        self, mock_pid_xml_utils, mock_xsd_tree, mock_create_tree_from_xpath
    ):
        """test_xsdtree_build_tree_called"""
        mock_data_init_content = deepcopy(self.mock_data.content)
        data_utils.set_pid_value_for_data(**self.kwargs)

        mock_xsd_tree.build_tree.assert_called_with(mock_data_init_content)

    @patch.object(data_utils, "create_tree_from_xpath")
    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_create_tree_from_xpath_called(
        self, mock_pid_xml_utils, mock_xsd_tree, mock_create_tree_from_xpath
    ):
        """test_create_tree_from_xpath_called"""
        mock_pid_xpath = MagicMock()
        mock_target_namespace = MagicMock()
        mock_xml_tree = MagicMock()
        mock_pid_xml_utils.get_xpath_with_target_namespace.return_value = (
            mock_pid_xpath
        )
        mock_pid_xml_utils.get_target_namespace_for_xsd_string.return_value = (
            mock_target_namespace
        )
        mock_xsd_tree.build_tree.return_value = mock_xml_tree

        data_utils.set_pid_value_for_data(**self.kwargs)

        mock_create_tree_from_xpath.assert_called_with(
            mock_pid_xpath, mock_xml_tree, mock_target_namespace
        )

    @patch.object(data_utils, "create_tree_from_xpath")
    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_set_value_at_xpath_called(
        self, mock_pid_xml_utils, mock_xsd_tree, mock_create_tree_from_xpath
    ):
        """test_set_value_at_xpath_called"""
        mock_pid_xpath = MagicMock()
        mock_target_namespace = MagicMock()
        mock_xml_tree = MagicMock()
        mock_pid_xml_utils.get_xpath_with_target_namespace.return_value = (
            mock_pid_xpath
        )
        mock_pid_xml_utils.get_target_namespace_for_xsd_string.return_value = (
            mock_target_namespace
        )
        mock_create_tree_from_xpath.return_value = mock_xml_tree

        data_utils.set_pid_value_for_data(**self.kwargs)

        mock_pid_xml_utils.set_value_at_xpath.assert_called_with(
            mock_xml_tree,
            mock_pid_xpath,
            self.kwargs["pid_value"],
            mock_target_namespace,
        )

    @patch.object(data_utils, "create_tree_from_xpath")
    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_xsdtree_to_string_called(
        self, mock_pid_xml_utils, mock_xsd_tree, mock_create_tree_from_xpath
    ):
        """test_xsdtree_to_string_called"""
        mock_xml_tree = MagicMock()
        mock_create_tree_from_xpath.return_value = mock_xml_tree

        data_utils.set_pid_value_for_data(**self.kwargs)

        mock_xsd_tree.tostring.assert_called_with(mock_xml_tree)

    @patch.object(data_utils, "create_tree_from_xpath")
    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_convert_to_file_called(
        self, mock_pid_xml_utils, mock_xsd_tree, mock_create_tree_from_xpath
    ):
        """test_convert_to_file_called"""
        data_utils.set_pid_value_for_data(**self.kwargs)

        self.kwargs["data"].convert_to_file.assert_called()

    @patch.object(data_utils, "create_tree_from_xpath")
    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_convert_to_dict_called(
        self, mock_pid_xml_utils, mock_xsd_tree, mock_create_tree_from_xpath
    ):
        """test_convert_to_dict_called"""
        data_utils.set_pid_value_for_data(**self.kwargs)

        self.kwargs["data"].convert_to_dict.assert_called()

    @patch.object(data_utils, "create_tree_from_xpath")
    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_succesful_execution_returns_none(
        self, mock_pid_xml_utils, mock_xsd_tree, mock_create_tree_from_xpath
    ):
        """test_succesful_execution_returns_none"""
        self.assertIsNone(data_utils.set_pid_value_for_data(**self.kwargs))


class TestSetPidValueForDataForJsonData(TestCase):
    """Unit tests for `set_pid_value_for_data` function, limiting to JSON data."""

    def setUp(self):
        """setUp"""
        template_format = Template.JSON
        self.mock_data = MagicMock()
        self.mock_data.template.format = template_format

        self.kwargs = {
            "data": self.mock_data,
            "pid_path": "mock_pid_path",
            "pid_value": "pid_value",
        }

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "load_json_string")
    @patch.object(data_utils, "json")
    def test_json_loads_called(
        self, mock_json, mock_load_json_string, mock_pid_json_utils
    ):
        """test_json_loads_called"""
        mock_data_init_content = deepcopy(self.mock_data.content)
        data_utils.set_pid_value_for_data(**self.kwargs)

        mock_load_json_string.assert_called_with(mock_data_init_content)

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "load_json_string")
    @patch.object(data_utils, "json")
    def test_set_value_at_dict_path_called(
        self, mock_json, mock_load_json_string, mock_pid_json_utils
    ):
        """test_set_value_at_dict_path_called"""
        mock_json_content = MagicMock()
        mock_load_json_string.return_value = mock_json_content

        data_utils.set_pid_value_for_data(**self.kwargs)

        mock_pid_json_utils.set_value_at_dict_path.assert_called_with(
            mock_json_content,
            self.kwargs["pid_path"],
            self.kwargs["pid_value"],
        )

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "load_json_string")
    @patch.object(data_utils, "json")
    def test_json_dumps_called(
        self, mock_json, mock_load_json_string, mock_pid_json_utils
    ):
        """test_json_dumps_called"""
        mock_json_content = MagicMock()
        mock_pid_json_utils.set_value_at_dict_path.return_value = (
            mock_json_content
        )

        data_utils.set_pid_value_for_data(**self.kwargs)

        mock_json.dumps.assert_called_with(mock_json_content)

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "load_json_string")
    @patch.object(data_utils, "json")
    def test_convert_to_file_called(
        self, mock_json, mock_load_json_string, mock_pid_json_utils
    ):
        """test_convert_to_file_called"""
        data_utils.set_pid_value_for_data(**self.kwargs)

        self.kwargs["data"].convert_to_file.assert_called()

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "load_json_string")
    @patch.object(data_utils, "json")
    def test_convert_to_dict_called(
        self, mock_json, mock_load_json_string, mock_pid_json_utils
    ):
        """test_convert_to_dict_called"""
        data_utils.set_pid_value_for_data(**self.kwargs)

        self.kwargs["data"].convert_to_dict.assert_called()

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "load_json_string")
    @patch.object(data_utils, "json")
    def test_succesful_execution_returns_none(
        self, mock_json, mock_load_json_string, mock_pid_json_utils
    ):
        """test_succesful_execution_returns_none"""
        self.assertIsNone(data_utils.set_pid_value_for_data(**self.kwargs))


class TestSetPidValueForDataForUnsupportedFormats(TestCase):
    """Unit tests for `set_pid_value_for_data` function, limiting to unsupported
    formats."""

    def setUp(self):
        """setUp"""
        template_format = "mock_unsupported_format"
        self.mock_data = MagicMock()
        self.mock_data.template.format = template_format

        self.kwargs = {
            "data": self.mock_data,
            "pid_path": "mock_pid_path",
            "pid_value": "pid_value",
        }

    @patch.object(data_utils, "logger")
    def test_logger_error_called(self, mock_logger):
        """test_logger_error_called"""
        with self.assertRaises(exceptions.PidCreateError):
            data_utils.set_pid_value_for_data(**self.kwargs)

        mock_logger.error.assert_called()


class TestGetPidValueForDataForXMLData(TestCase):
    """Unit tests for `get_pid_value_for_data` function, limiting to XML data."""

    def setUp(self):
        """setUp"""
        template_format = Template.XSD
        self.mock_data = MagicMock()
        self.mock_data.template.format = template_format

        self.kwargs = {"data": self.mock_data, "pid_path": "mock_pid_path"}

    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_get_xpath_with_target_namespace_called(
        self, mock_pid_xml_utils, mock_xsd_tree
    ):
        """test_get_xpath_with_target_namespace_called"""
        data_utils.get_pid_value_for_data(**self.kwargs)

        mock_pid_xml_utils.get_xpath_with_target_namespace.assert_called_with(
            mock_pid_xml_utils.get_xpath_from_dot_notation(
                self.kwargs["pid_path"]
            ),
            self.kwargs["data"].template.content,
        )

    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_get_target_namespace_for_xsd_string_called(
        self, mock_pid_xml_utils, mock_xsd_tree
    ):
        """test_get_target_namespace_for_xsd_string_called"""
        data_utils.get_pid_value_for_data(**self.kwargs)

        mock_pid_xml_utils.get_target_namespace_for_xsd_string.assert_called_with(
            self.kwargs["data"].template.content
        )

    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_xsd_tree_build_tree_called(
        self, mock_pid_xml_utils, mock_xsd_tree
    ):
        """test_xsd_tree_build_tree_called"""
        mock_data_init_content = self.kwargs["data"].content
        data_utils.get_pid_value_for_data(**self.kwargs)

        mock_xsd_tree.build_tree.assert_called_with(mock_data_init_content)

    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_get_value_at_xpath_called(
        self, mock_pid_xml_utils, mock_xsd_tree
    ):
        """test_get_value_at_xpath_called"""
        mock_xml_tree = MagicMock()
        mock_pid_xpath = MagicMock()
        mock_target_namespace = MagicMock()

        mock_pid_xml_utils.get_xpath_with_target_namespace.return_value = (
            mock_pid_xpath
        )
        mock_pid_xml_utils.get_target_namespace_for_xsd_string.return_value = (
            mock_target_namespace
        )
        mock_xsd_tree.build_tree.return_value = mock_xml_tree

        data_utils.get_pid_value_for_data(**self.kwargs)

        mock_pid_xml_utils.get_value_at_xpath.assert_called_with(
            mock_xml_tree, mock_pid_xpath, mock_target_namespace
        )

    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_get_value_at_xpath_error_call_can_create_value_at_xpath(
        self, mock_pid_xml_utils, mock_xsd_tree
    ):
        """test_get_value_at_xpath_error_call_can_create_value_at_xpath"""
        mock_data_init_content = self.kwargs["data"].content
        mock_pid_xpath = MagicMock()

        mock_pid_xml_utils.get_xpath_with_target_namespace.return_value = (
            mock_pid_xpath
        )
        mock_pid_xml_utils.get_value_at_xpath.side_effect = (
            xml_utils_exceptions.XPathError(
                "mock_get_value_at_xpath_exception"
            )
        )

        data_utils.get_pid_value_for_data(**self.kwargs)

        mock_pid_xml_utils.can_create_value_at_xpath.assert_called_with(
            mock_data_init_content,
            self.kwargs["data"].template.content,
            mock_pid_xpath,
            "http://sample_pid.org",
        )

    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_can_create_value_at_xpath_false_raises_pid_create_error(
        self, mock_pid_xml_utils, mock_xsd_tree
    ):
        """test_can_create_value_at_xpath_false_raises_pid_create_error"""
        mock_pid_xml_utils.get_value_at_xpath.side_effect = (
            xml_utils_exceptions.XPathError(
                "mock_get_value_at_xpath_exception"
            )
        )
        mock_pid_xml_utils.can_create_value_at_xpath.return_value = False

        with self.assertRaises(exceptions.PidCreateError):
            data_utils.get_pid_value_for_data(**self.kwargs)

    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_get_value_at_xpath_error_returns_none(
        self, mock_pid_xml_utils, mock_xsd_tree
    ):
        """test_get_value_at_xpath_error_returns_none"""
        mock_pid_xml_utils.get_value_at_xpath.side_effect = (
            xml_utils_exceptions.XPathError(
                "mock_get_value_at_xpath_exception"
            )
        )
        mock_pid_xml_utils.can_create_value_at_xpath.return_value = True

        self.assertIsNone(data_utils.get_pid_value_for_data(**self.kwargs))

    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_successful_executiopn_returns_pid_value(
        self, mock_pid_xml_utils, mock_xsd_tree
    ):
        """test_successful_executiopn_returns_pid_value"""
        mock_pid_value = "mock_pid_value"

        mock_pid_xml_utils.get_value_at_xpath.return_value = mock_pid_value

        self.assertEqual(
            data_utils.get_pid_value_for_data(**self.kwargs), mock_pid_value
        )

    @patch.object(data_utils, "XSDTree")
    @patch.object(data_utils, "pid_xml_utils")
    def test_pid_value_truncated_if_ending_with_slash(
        self, mock_pid_xml_utils, mock_xsd_tree
    ):
        """test_pid_value_truncated_if_ending_with_slash"""
        mock_pid_value = "mock_pid_value/"

        mock_pid_xml_utils.get_value_at_xpath.return_value = mock_pid_value

        self.assertEqual(
            data_utils.get_pid_value_for_data(**self.kwargs),
            mock_pid_value[:-1],
        )


class TestGetPidValueForDataForJsonData(TestCase):
    """Unit tests for `get_pid_value_for_data` function, limiting to JSON data."""

    def setUp(self):
        """setUp"""
        template_format = Template.JSON
        self.mock_data = MagicMock()
        self.mock_data.template.format = template_format

        self.kwargs = {
            "data": self.mock_data,
            "pid_path": "mock_pid_path",
        }

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "pid_dict_utils")
    @patch.object(data_utils, "load_json_string")
    def test_json_loads_called(
        self, mock_load_json_string, mock_pid_dict_utils, mock_pid_json_utils
    ):
        """test_json_loads_called"""
        mock_data_init_content = self.kwargs["data"].content
        data_utils.get_pid_value_for_data(**self.kwargs)

        mock_load_json_string.assert_called_with(mock_data_init_content)

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "pid_dict_utils")
    @patch.object(data_utils, "load_json_string")
    def test_get_value_from_dot_notation_called(
        self, mock_load_json_string, mock_pid_dict_utils, mock_pid_json_utils
    ):
        """test_get_value_from_dot_notation_called"""
        mock_json_content = MagicMock()
        mock_load_json_string.return_value = mock_json_content

        data_utils.get_pid_value_for_data(**self.kwargs)

        mock_pid_dict_utils.get_value_from_dot_notation.assert_called_with(
            mock_json_content, self.kwargs["pid_path"]
        )

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "pid_dict_utils")
    @patch.object(data_utils, "load_json_string")
    def test_can_create_value_at_dict_path_called(
        self, mock_load_json_string, mock_pid_dict_utils, mock_pid_json_utils
    ):
        """test_cannot_create_value_at_dict_path_raises_pid_create_error"""
        mock_json_content = MagicMock()

        mock_pid_dict_utils.get_value_from_dot_notation.return_value = None
        mock_load_json_string.return_value = mock_json_content

        data_utils.get_pid_value_for_data(**self.kwargs)

        mock_pid_json_utils.can_create_value_at_dict_path.assert_called_with(
            mock_json_content,
            self.kwargs["data"].template.content,
            self.kwargs["pid_path"],
            "http://sample_pid.org",
        )

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "pid_dict_utils")
    @patch.object(data_utils, "load_json_string")
    def test_cannot_create_value_at_dict_path_raises_pid_create_error(
        self, mock_load_json_string, mock_pid_dict_utils, mock_pid_json_utils
    ):
        """test_cannot_create_value_at_dict_path_raises_pid_create_error"""
        mock_pid_dict_utils.get_value_from_dot_notation.return_value = None
        mock_pid_json_utils.can_create_value_at_dict_path.return_value = False

        with self.assertRaises(exceptions.PidCreateError):
            data_utils.get_pid_value_for_data(**self.kwargs)

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "pid_dict_utils")
    @patch.object(data_utils, "load_json_string")
    def test_successful_execution_returns_pid_value(
        self, mock_load_json_string, mock_pid_dict_utils, mock_pid_json_utils
    ):
        """test_successful_execution_returns_pid_value"""
        mock_pid_value = "mock_pid_value"

        mock_pid_dict_utils.get_value_from_dot_notation.return_value = (
            mock_pid_value
        )

        self.assertEqual(
            data_utils.get_pid_value_for_data(**self.kwargs), mock_pid_value
        )

    @patch.object(data_utils, "pid_json_utils")
    @patch.object(data_utils, "pid_dict_utils")
    @patch.object(data_utils, "load_json_string")
    def test_pid_value_truncated_if_ending_with_slash(
        self, mock_load_json_string, mock_pid_dict_utils, mock_pid_json_utils
    ):
        """test_pid_value_truncated_if_ending_with_slash"""
        mock_pid_value = "mock_pid_value/"

        mock_pid_dict_utils.get_value_from_dot_notation.return_value = (
            mock_pid_value
        )

        self.assertEqual(
            data_utils.get_pid_value_for_data(**self.kwargs),
            mock_pid_value[:-1],
        )


class TestGetPidValueForDataForUnsupportedFormats(TestCase):
    """Unit tests for `get_pid_value_for_data` function, limiting to unsupported
    formats."""

    def setUp(self):
        """setUp"""
        template_format = "mock_unsupported_format"
        self.mock_data = MagicMock()
        self.mock_data.template.format = template_format

        self.kwargs = {
            "data": self.mock_data,
            "pid_path": "pid_path",
        }

    @patch.object(data_utils, "logger")
    def test_logger_error_called(self, mock_logger):
        """test_logger_error_called"""
        with self.assertRaises(exceptions.InvalidPidError):
            data_utils.get_pid_value_for_data(**self.kwargs)

        mock_logger.error.assert_called()



""" Unit tests for core_linked_records_app.system.pid_path.api
"""
from unittest import TestCase
from unittest.mock import patch, Mock

from core_linked_records_app.components.pid_path.models import PidPath
from core_linked_records_app.system.pid_path import (
    api as pid_path_system_api,
)


class TestGetPidPathByTemplate(TestCase):
    """Test get_pid_path_by_template method"""

    @patch(
        "core_linked_records_app.system.pid_path.api.PidPath.get_by_template"
    )
    def test_get_by_template_is_called(self, mock_get_by_template):
        """Test get_by_template is called"""
        mock_template = "mock_template"

        pid_path_system_api.get_pid_path_by_template(mock_template)
        mock_get_by_template.assert_called_with(mock_template)

    @patch("core_linked_records_app.system.pid_path.api.PidPath.__new__")
    @patch(
        "core_linked_records_app.system.pid_path.api.PidPath.get_by_template"
    )
    def test_get_by_template_none_returns_pid_path(
        self, mock_get_by_template, mock_pid_path
    ):
        """If get_by_template returns None, the PidPath returned is the default one."""
        expected_result = Mock(spec=PidPath)
        mock_get_by_template.return_value = None
        mock_pid_path.return_value = expected_result

        result = pid_path_system_api.get_pid_path_by_template("mock_template")
        self.assertEqual(result, expected_result)

    @patch(
        "core_linked_records_app.system.pid_path.api.PidPath.get_by_template"
    )
    def test_returns_get_by_template_if_not_none(self, mock_get_by_template):
        """If get_by_template is not None, the PidPath returned is get_by_template."""
        expected_result = "mock_result"
        mock_get_by_template.return_value = expected_result

        result = pid_path_system_api.get_pid_path_by_template("mock_template")
        self.assertEqual(result, expected_result)



""" Test settings
"""

SECRET_KEY = "fake-key"

INSTALLED_APPS = [
    # Django apps
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.sites",
    "django.contrib.staticfiles",
    # Extra apps
    "defender",
    "django_celery_beat",
    "drf_spectacular",
    "menu",
    # Local apps
    "xml_utils",
    "core_main_app",
    "core_explore_common_app",
    "core_explore_oaipmh_app",
    "core_federated_search_app",
    "core_oaipmh_common_app",
    "core_oaipmh_harvester_app",
    "core_linked_records_app",
    "tests",
]

# IN-MEMORY TEST DATABASE
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
        "USER": "",
        "PASSWORD": "",
        "HOST": "",
        "PORT": "",
    },
}

MIDDLEWARE = (
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
)

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": ["templates"],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "core_main_app.utils.custom_context_processors.domain_context_processor",  # Needed by any curator app
                "django.template.context_processors.i18n",
            ],
        },
    },
]

CUSTOM_NAME = "mock_custom_name"

ROOT_URLCONF = "tests.test_urls"

STATIC_URL = "/static/"

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

CELERYBEAT_SCHEDULER = "django_celery_beat.schedulers:DatabaseScheduler"
SERVER_URI = "http://mock-cdcs.com"

PID_PATH = "mock.pid"

ID_PROVIDER_PREFIXES = ["mock_cdcs"]

MONGODB_INDEXING = False
MONGODB_ASYNC_SAVE = False

ENABLE_SAML2_SSO_AUTH = False

CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT = False



""" Url router for the core linked records app
"""

from django.urls import re_path

from core_linked_records_app.rest.blob import views as blob_views
from core_linked_records_app.rest.pid import views as pid_views
from core_linked_records_app.rest.pid_settings import views as settings_views
from core_linked_records_app.rest.pid_path import views as pid_path_views
from core_linked_records_app.rest.providers import views as providers_views
from core_linked_records_app.rest.query import views as query_views

urlpatterns = [
    re_path(
        r"^query$",
        query_views.RetrieveQueryPidListView.as_view(),
        name="core_linked_records_app_query_pid",
    ),
]

urlpatterns += [
    re_path(
        r"^settings/$",
        settings_views.PidSettingsView.as_view(),
        name="core_linked_records_app_settings",
    ),
    re_path(
        r"^settings/path/$",
        pid_path_views.PidPathListView.as_view(),
        name="core_linked_records_app_settings_path_list",
    ),
    re_path(
        r"^settings/path/(?P<pk>[0-9]+)/$",
        pid_path_views.PidPathDetailView.as_view(),
        name="core_linked_records_app_settings_path_detail",
    ),
    re_path(
        r"^retrieve-list-pid",
        pid_views.RetrieveListPIDView.as_view(),
        name="core_linked_records_retrieve_list_pid",
    ),
    re_path(
        r"^retrieve-data-pid",
        pid_views.RetrieveDataPIDView.as_view(),
        name="core_linked_records_retrieve_data_pid",
    ),
    re_path(
        r"^retrieve-blob-pid",
        pid_views.RetrieveBlobPIDView.as_view(),
        name="core_linked_records_retrieve_blob_pid",
    ),
    re_path(
        r"^upload-blob-pid",
        blob_views.BlobUploadWithPIDView.as_view(),
        name="core_linked_records_upload_blob_pid",
    ),
    re_path(
        r"^(?P<provider>[^/]+)/(?P<record>.*)$",
        providers_views.ProviderRecordView.as_view(),
        name="core_linked_records_provider_record",
    ),
]



""" Unit tests for data_xml_renderer packages.
"""
from unittest import TestCase

from core_linked_records_app.rest.data.renderers.data_xml_renderer import (
    DataXmlRenderer,
)


class TestDataXmlRendererRender(TestCase):
    """Unit tests for `DataXmlRenderer.render` method."""

    def test_xml_renderer_returns_data_content(self):
        """test_xml_renderer_returns_data_content"""
        data_content = "<root></root>"
        mock_data = {"xml_content": data_content}
        renderer = DataXmlRenderer()
        result = renderer.render(mock_data)
        self.assertEqual(result, data_content)



""" Unit tests for core_linked_records_app.components.pid_settings.models
"""
from unittest import TestCase
from unittest.mock import patch

from django.core.exceptions import ObjectDoesNotExist

from core_linked_records_app.components.pid_settings.models import PidSettings
from core_main_app.commons import exceptions


class TestPidSettingsGet(TestCase):
    """Test Pid Settings Get"""

    @patch.object(PidSettings, "objects")
    def test_pid_settings_first_does_not_exist_returns_none(
        self, mock_pid_settings
    ):
        """test_pid_settings_first_does_not_exist_returns_none"""
        mock_pid_settings.first.side_effect = ObjectDoesNotExist()

        self.assertIsNone(PidSettings.get())

    @patch.object(PidSettings, "objects")
    def test_pid_settings_first_failure_raises_model_error(
        self, mock_pid_settings
    ):
        """test_pid_settings_first_failure_raises_model_error"""
        mock_pid_settings.first.side_effect = Exception(
            "mock_pid_settings_first_exception"
        )

        with self.assertRaises(exceptions.ModelError):
            PidSettings.get()

    @patch.object(PidSettings, "objects")
    def test_returns_pid_settings_first_output(self, mock_pid_settings):
        """test_returns_pid_settings_first_output"""
        expected_result = "mock_pid_settings"
        mock_pid_settings.first.return_value = expected_result

        self.assertEqual(PidSettings.get(), expected_result)



""" Exceptions for core_linked_records_app.
"""
from core_main_app.commons.exceptions import CoreError


class PidCreateError(CoreError):
    """Exception raised while creating a PID."""

    def __init__(self, message):
        """Initialize exception

        Args:
            message (str): Error message
        """
        super().__init__(message)


class PidResolverError(CoreError):
    """Exception raised when resolving a PID."""

    def __init__(self, message):
        """Initialize exception

        Args:
            message (str): Error message
        """
        super().__init__(message)


class InvalidProviderError(CoreError):
    """Exception raised when a provider is not valid."""

    def __init__(self, message):
        """Initialize exception

        Args:
            message (str): Error message
        """
        super().__init__(message)


class InvalidPidError(CoreError):
    """Exception raised when a PID is not valid."""

    def __init__(self, message):
        """Initialize exception

        Args:
            message (str): Error message
        """
        super().__init__(message)


class InvalidPrefixError(CoreError):
    """Exception raised if an invalid prefix is detected."""

    def __init__(self, message):
        """Initialize exception

        Args:
            message (str): Error message
        """
        super().__init__(message)


class InvalidRecordError(CoreError):
    """Exception raised if an invalid record is detected."""

    def __init__(self, message):
        """Initialize exception

        Args:
            message (str): Error message
        """
        super().__init__(message)



""" Utilities to manipulate dictionaries
"""
from django.core.exceptions import ValidationError

from core_main_app.commons.exceptions import QueryError
from core_main_app.utils.query.mongo.prepare import sanitize_value


def validate_dot_notation(value):
    """Validate value used for dot notation to avoid Mongo injection.

    Args:
        value: The value to sanitize

    Raises:
        ValidationError - If the query cannot be sanitized
    """
    try:
        sanitize_value(value)
    except QueryError as error:
        raise ValidationError(f"Query error: {str(error)}")
    except Exception as exception:
        raise ValidationError(f"Unexpected error: {str(exception)}")


def get_value_from_dot_notation(dictionary, dot_notation):
    """Retrieve dictionary content given a dot notation path.

    Params:
        dictionary:
        key_list:

    Returns:
    """
    # Split dot_notation except if it is None or ''
    key_list = dot_notation.split(".") if dot_notation else []

    while len(key_list) > 0:
        key = key_list.pop(0)

        if key in dictionary.keys():
            return get_value_from_dot_notation(
                dictionary[key], ".".join(key_list)
            )
        return None

    return dictionary


def is_dot_notation_in_dictionary(dictionary, dot_notation):
    """Given a dot notation path, find if it is present in a dictionary.

    Params:
        dictionary:
        dot_notation:

    Returns:
    """
    # Split dot_notation except if it is None or ''
    key_list = dot_notation.split(".") if dot_notation else []

    while len(key_list) > 0:
        key = key_list.pop(0)

        if key in dictionary.keys():
            return is_dot_notation_in_dictionary(
                dictionary[key], ".".join(key_list)
            )
        return False

    return True



""" JSON utilities functions.
"""
import logging

from core_main_app.utils.json_utils import validate_json_data

logger = logging.getLogger(__name__)


def set_value_at_dict_path(dictionary, dot_notation, value):
    """Create a value at a given dot notation path

    Params:
        dictionary:
        dot_notation:
        value:

    Returns:
    """
    # Split dot_notation except if it is None or ''
    key_list = dot_notation.split(".") if dot_notation else []

    # If the key list contains only one key, assign the value to that key and return the
    # updated dictionary.
    if len(key_list) == 1:
        dictionary[key_list.pop(0)] = value
        return dictionary

    if len(key_list) < 1:
        raise KeyError(f"Cannot set value at path {dot_notation}")

    key = key_list.pop(0)
    dictionary[key] = set_value_at_dict_path(
        dictionary[key] if key in dictionary.keys() else {},
        ".".join(key_list),
        value,
    )
    return dictionary


def can_create_value_at_dict_path(json_dict, template_dict, dict_path, value):
    """Evaluate if a value can be set in an JSON dict at a given dot notation path

    Params:
        json_dict:
        template_dict:
        dict_path:
        value:

    Returns:
        bool - True if the value can be created, False otherwise.
    """
    try:
        # Replace the current by the modified tree (containing mock PID) and
        # force document PID to be regenerated.
        set_value_at_dict_path(json_dict, dict_path, value)

        validation_error = validate_json_data(json_dict, template_dict)
        if validation_error is not None:
            raise Exception(f"Error while validating JSON: {validation_error}")

        return True
    except Exception as exc:  # pylint: disable=broad-except
        logger.info(
            "Function 'can_create_value_at_dict_path' raised %s", str(exc)
        )
        return False



""" Administration views
"""

from django.urls import reverse
from django.views.generic import View

from core_linked_records_app.components.pid_settings import (
    api as pid_settings_api,
)
from core_linked_records_app.components.pid_path import api as pid_path_api
from core_linked_records_app.settings import SERVER_URI
from core_linked_records_app.utils.pid import get_pid_settings_dict
from core_main_app.utils.rendering import admin_render


class PidSettingsView(View):
    """View to display PID settings in the admin part"""

    def get(self, request):
        """HTTP GET method"""
        context = {}

        assets = {
            "js": [
                {
                    "path": "core_linked_records_app/admin/js/pid_settings/auto_set_pid.js",
                    "is_raw": False,
                }
            ],
            "css": [
                "core_linked_records_app/admin/css/pid_settings.css",
            ],
        }

        pid_path_settings = {}

        # Retrieve existing PidPath
        try:
            for pid_path_item in pid_path_api.get_all(request):
                pid_path_settings[pid_path_item.template.display_name] = {
                    "path": pid_path_item.path,
                    "edit_url": reverse(
                        "admin:core_linked_records_app_pidpath_change",
                        args=(pid_path_item.pk,),
                    ),
                }

            pid_settings = pid_settings_api.get(request.user)
            response_dict = get_pid_settings_dict(pid_settings)
            response_dict["path"] = {
                "default": {"path": response_dict["path"]}
            }
            response_dict["path"].update(pid_path_settings)

            response_dict["add_path_url"] = reverse(
                "admin:core_linked_records_app_pidpath_add"
            )

            record_sample_url = reverse(
                "core_linked_records_provider_record",
                kwargs={
                    "provider": response_dict["system_name"],
                    "record": f"{response_dict['prefixes'][0]}/record",
                },
            )
            response_dict["sample_url"] = f"{SERVER_URI}{record_sample_url}"

            context.update({"pid_settings": response_dict})

            return admin_render(
                request,
                "core_linked_records_app/admin/pid_settings.html",
                assets=assets,
                context=context,
            )
        except Exception as exc:
            return admin_render(
                request,
                "core_linked_records_app/admin/pid_settings_error.html",
                context={
                    "error": "An error occured while retrieving the PID settings. "
                    "Please contact an administrator for more information. Exception: "
                    f"{str(exc)}."
                },
            )



""" Mock classes for objects used within core_linked_records_app
"""
from unittest.mock import Mock

from rest_framework import status


class MockModule(Mock):
    """Mock Module"""

    get_by_id_return_value = "MockModule.get_by_id"

    def get_by_id(
        self, *args, **kwargs  # noqa, pylint: disable=unused-argument
    ):
        """get_by_id

        Returns:
        """
        return self.get_by_id_return_value


class MockLocalId(Mock):
    """Mock Local Id"""

    record_name = "mock_record_name"
    record_object_class = "mock.record.object.class"
    record_object_id = "mock_object_id"


class MockDocument(Mock):
    """Mock Document"""

    id = 1234
    pk = 1234


class MockData(MockDocument):
    """Mock Data"""

    dict_content = "mock_dict_content"


class MockTemplate(Mock):
    """Mock Template"""

    display_name = None


class MockPidSettings(Mock):
    """Mock Pid Settings"""

    auto_set_pid = True


class MockPidPath(MockDocument):
    """Mock PidPath object"""

    path = "mock.path"
    template = 1234


class MockResponse(Mock):
    """Mock Response"""

    status_code = status.HTTP_200_OK
    data = {}
    content = "{}"
    json_data = None

    def json(self):
        """json"""
        return self.json_data


class MockSession(Mock):
    """MockSession"""

    session_key = "mock_session"


class MockRequest(Mock):
    """Mock Request"""

    user = None
    session = MockSession()


class MockProviderManager(Mock):
    """Mock Provider Manager"""

    provider_lookup_url = "mock_provider_url"
    create_exc = None
    create_result = None
    update_exc = None
    update_result = None
    get_exc = None
    get_result = None
    delete_exc = None
    delete_result = None

    def create(self, *args, **kwargs):  # noqa, pylint: disable=unused-argument
        """create"""
        if self.create_exc and issubclass(self.create_exc, Exception):
            raise self.create_exc  # pylint: disable=raising-bad-type

        return self.create_result

    def update(self, *args, **kwargs):  # noqa, pylint: disable=unused-argument
        """update"""
        if self.update_exc and issubclass(self.update_exc, Exception):
            raise self.update_exc  # pylint: disable=raising-bad-type

        return self.update_result

    def get(self, *args, **kwargs):  # noqa, pylint: disable=unused-argument
        """get"""
        if self.get_exc and issubclass(self.get_exc, Exception):
            raise self.get_exc  # pylint: disable=raising-bad-type

        return self.get_result

    def delete(self, *args, **kwargs):  # noqa, pylint: disable=unused-argument
        """delete"""
        if self.delete_exc and issubclass(self.delete_exc, Exception):
            raise self.delete_exc  # pylint: disable=raising-bad-type

        return self.delete_result


class MockQuery(Mock):
    """Mock Query"""

    class MockTemplates(Mock):
        """Mock Templates"""

        templates = []

        def all(self):
            """all"""
            return self.templates

    data_sources = []
    content = ""
    templates = MockTemplates()


class MockDataSource(Mock):
    """Mock Data Source"""

    query_options = {}
    order_by_field = ""
    capabilities = {}


class MockAuthentication(Mock):
    """Mock Authentication"""

    auth_type = ""


class MockSerializer(Mock):
    """Mock Serializer"""

    data = {}
    is_valid_exc = None
    is_valid_result = True
    update_exc = None
    update_result = None

    def is_valid(self):
        """is_valid"""
        if self.is_valid_exc and issubclass(self.is_valid_exc, Exception):
            raise self.is_valid_exc  # pylint: disable=raising-bad-type
        return self.is_valid_result

    def update(self, *args):  # noqa, pylint: disable=unused-argument
        """update"""
        if self.update_exc and issubclass(self.update_result, Exception):
            raise self.update_exc  # pylint: disable=raising-bad-type
        return self.update_result


class MockInstance(Mock):
    """Mock Instance"""

    endpoint = "mock_endpoint"
    access_token = "mock_access_token"


class MockBlob(Mock):
    """Mock Blob"""

    blob = Mock()
    filename = "mock_filename"



""" REST views for the query API
"""

from core_explore_common_app.rest.query.views import build_local_query
from core_linked_records_app import settings
from core_linked_records_app.components.pid_path import api as pid_path_api
from core_linked_records_app.utils.dict import get_value_from_dot_notation
from core_linked_records_app.utils.pid import is_valid_pid_value
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons.exceptions import ApiError, CoreError
from core_main_app.components.data import api as data_api


def build_pid_query(query, build_fn, request):
    """Build the query by adding an extra filter to limit to document with
    PID fields.

    Args:
        query:
        build_fn:
        request:

    Returns:
        Query with additional PID filter
    """
    query = build_fn(query)
    query = query["$and"] if "$and" in query.keys() else [query]

    # build PID query and append the raw query to it
    pid_path_list = [
        pid_path_object.path
        for pid_path_object in pid_path_api.get_all(request)
    ] + [settings.PID_PATH]
    pid_query = {
        "$and": [
            {
                "$or": [
                    {f"dict_content.{pid_path}": {"$exists": 1}}
                    for pid_path in pid_path_list
                ]
            }
        ]
        + query
    }

    return pid_query


def execute_pid_query(json_query, build_fn, execute_fn, request):
    """Build and execute a query given specific build and execute functions.

    Args:
        json_query:
        build_fn:
        execute_fn:
        request:

    Raises:
        AccessControlError
        ApiError

    Returns:
        Output of the execute function provided
    """
    try:
        # prepare query
        pid_query = build_pid_query(
            query=json_query, build_fn=build_fn, request=request
        )
        # execute query
        return execute_fn(pid_query, request)
    except AccessControlError as acl_error:
        raise AccessControlError(str(acl_error))
    except Exception as api_exception:
        raise ApiError(str(api_exception))


def execute_local_query(raw_query, request):
    """Execute the raw query in database

    Args:

        raw_query: Query to execute
        request:

    Returns:
        Results of the query
    """
    pid_list = list()
    data_list = data_api.execute_json_query(raw_query, request.user)

    for data in data_list:
        pid_path_object = pid_path_api.get_by_template(
            data.template, request.user
        )
        pid_path = pid_path_object.path

        data_pid = get_value_from_dot_notation(
            data.get_dict_content(),
            pid_path,
        )

        if not is_valid_pid_value(
            data_pid, settings.ID_PROVIDER_SYSTEM_NAME, settings.PID_FORMAT
        ):
            continue

        pid_list.append(data_pid)

    return pid_list


def execute_local_pid_query(json_query, request):
    """Execute a query to retrieve PIDs on data from a local data source.

    Args:
        json_query:
        request:

    Returns:
        Output of the `execute_local_query` function
    """
    return execute_pid_query(
        json_query, build_local_query, execute_local_query, request
    )


def execute_oaipmh_query(raw_query, request):
    """Execute the raw query in database

    Args:
        raw_query: Query to execute
        request:

    Returns:
        Results of the query
    """
    if "core_oaipmh_harvester_app" not in settings.INSTALLED_APPS:
        raise CoreError(
            "Missing dependency 'core_oaipmh_harvester_app' in INSTALLED_APPS"
        )

    from core_oaipmh_harvester_app.components.oai_record import (
        api as oai_record_api,
    )

    pid_list = list()
    data_list = oai_record_api.execute_json_query(raw_query, request.user)

    for data in data_list:
        pid_path_object = pid_path_api.get_by_template(
            data.harvester_metadata_format.template, request.user
        )
        pid_path = pid_path_object.path

        data_pid = get_value_from_dot_notation(
            data.get_dict_content(),
            pid_path,
        )

        if not data_pid:
            continue

        pid_list.append(data_pid)

    return pid_list


def execute_oaipmh_pid_query(json_query, request):
    """Execute a query to retrieve PIDs on data from a OAI-PMH data source.

    Args:
        json_query:
        request:

    Returns:
        Output of the `execute_oaipmh_query` function
    """
    if "core_explore_oaipmh_app" not in settings.INSTALLED_APPS:
        raise CoreError(
            "Missing dependency 'core_explore_oaipmh_app' in INSTALLED_APPS"
        )

    from core_explore_oaipmh_app.rest.query.views import build_oaipmh_query

    return execute_pid_query(
        json_query, build_oaipmh_query, execute_oaipmh_query, request
    )



""" Unit tests for core_linked_records_app.components.local_id.models
"""
from unittest import TestCase
from unittest.mock import patch

from django.core.exceptions import ObjectDoesNotExist
from django.db import IntegrityError

from core_linked_records_app.components.local_id.models import LocalId
from core_main_app.commons import exceptions


class TestGetByName(TestCase):
    """Test Get By Name"""

    @patch.object(LocalId, "objects")
    def test_local_id_get_does_not_exist_raises_does_not_exist_error(
        self, mock_objects
    ):
        """test_local_id_get_does_not_exist_raises_does_not_exist_error"""

        mock_objects.get.side_effect = ObjectDoesNotExist(
            "mock_objects_get_does_not_exist"
        )

        with self.assertRaises(exceptions.DoesNotExist):
            LocalId.get_by_name("mock_record_name")

    @patch.object(LocalId, "objects")
    def test_local_id_get_failure_raises_model_error(self, mock_objects):
        """test_local_id_get_failure_raises_model_error"""

        mock_objects.get.side_effect = Exception("mock_objects_get_exception")

        with self.assertRaises(exceptions.ModelError):
            LocalId.get_by_name("mock_record_name")

    @patch.object(LocalId, "objects")
    def test_returns_local_id_get_output(self, mock_objects):
        """test_returns_local_id_get_output"""

        expected_result = "mock_get_by_name"
        mock_objects.get.return_value = expected_result

        self.assertEqual(
            LocalId.get_by_name("mock_record_name"), expected_result
        )


class TestGetByClassAndId(TestCase):
    """Test Get By Class And Id"""

    @patch.object(LocalId, "objects")
    def test_local_id_get_does_not_exist_raises_does_not_exist_error(
        self, mock_objects
    ):
        """test_local_id_get_does_not_exist_raises_does_not_exist_error"""

        mock_objects.get.side_effect = ObjectDoesNotExist(
            "mock_objects_get_does_not_exist"
        )

        with self.assertRaises(exceptions.DoesNotExist):
            LocalId.get_by_class_and_id("mock_record_class", "mock_record_id")

    @patch.object(LocalId, "objects")
    def test_local_id_get_failure_raises_model_error(self, mock_objects):
        """test_local_id_get_failure_raises_model_error"""

        mock_objects.get.side_effect = Exception("mock_objects_get_exception")

        with self.assertRaises(exceptions.ModelError):
            LocalId.get_by_class_and_id("mock_record_class", "mock_record_id")

    @patch.object(LocalId, "objects")
    def test_returns_local_id_get_output(self, mock_objects):
        """test_returns_local_id_get_output"""

        expected_result = "mock_get_by_name"
        mock_objects.get.return_value = expected_result

        self.assertEqual(
            LocalId.get_by_class_and_id("mock_record_class", "mock_record_id"),
            expected_result,
        )


class TestUpdate(TestCase):
    """Test Update"""

    def setUp(self):
        """setUp"""

        self.mock_local_id = LocalId()

    @patch.object(LocalId, "save")
    def test_local_id_save_not_unique_error_raises_not_unique_error(
        self, mock_save
    ):
        """test_local_id_save_not_unique_error_raises_not_unique_error"""

        mock_save.side_effect = IntegrityError("mock_save_not_unique_error")

        with self.assertRaises(exceptions.NotUniqueError):
            LocalId.upsert(self.mock_local_id)

    @patch.object(LocalId, "save")
    def test_local_id_save_failure_raises_model_error(self, mock_save):
        """test_local_id_save_failure_raises_model_error"""

        mock_save.side_effect = Exception("mock_save_exception")

        with self.assertRaises(exceptions.ModelError):
            LocalId.upsert(self.mock_local_id)

    @patch.object(LocalId, "save")
    def test_returns_local_id_get_output(self, mock_save):
        """test_returns_local_id_get_output"""

        mock_save.return_value = None

        self.assertEqual(
            LocalId.upsert(self.mock_local_id),
            self.mock_local_id,
        )



""" System API to manage PidSettings objects.
"""
import logging

from core_linked_records_app.components.pid_settings.models import PidSettings
from core_main_app.commons.exceptions import ApiError

logger = logging.getLogger(__name__)


def upsert(pid_settings_object):
    """Insert or update the PidSettings provided as parameter.

    Args:
        pid_settings_object:

    Returns:
        Saved PidSettings object
    """
    try:
        pid_settings_object.save()
        return pid_settings_object
    except Exception as exc:
        error_message = "An unexpected error occurred while saving PidSettings"

        logger.error("%s: %s", error_message, str(exc))
        raise ApiError(f"{error_message}.") from exc


def get():
    """Retrieve the PidSettings object from DB.

    Returns:
        PidSettings object
    """
    try:
        return PidSettings.get()
    except Exception as exc:
        error_message = (
            "An unexpected error occurred while retrieving PidSettings"
        )

        logger.error("%s: %s", error_message, str(exc))
        raise ApiError(f"{error_message}.") from exc



""" Unit tests for core_linked_records_app.utils.path
"""
from unittest import TestCase


class TestGetApiPathFromObject(TestCase):
    """Test Get Api Path From Object"""

    def test_builtin_object_not_returned(self):
        """test_builtin_object_not_returned"""

        pass

    def test_path_to_api_returned(self):
        """test_path_to_api_returned"""

        pass



""" Unit tests for core_linked_records_app.utils.pid
"""
from unittest import TestCase
from unittest.mock import patch, Mock, MagicMock

from core_linked_records_app.utils import pid as pid_utils
from core_linked_records_app.utils.exceptions import (
    InvalidPrefixError,
    InvalidRecordError,
)
from core_linked_records_app.utils.providers import (
    AbstractIdProvider,
)
from tests import test_settings


class TestIsValidPidValue(TestCase):
    """Tests for is_valid_pid_value function"""

    @classmethod
    def setUpClass(cls) -> None:
        """Create test case global variables"""
        cls.mock_pid_provider_name = "mock_pid_provider_name"
        cls.mock_provider_url = "mock_provider_url"

        cls.mock_provider = Mock(spec=AbstractIdProvider)
        cls.mock_provider.provider_lookup_url = cls.mock_provider_url  # noqa

        cls.mock_pid_format = r"[a-z]+"

    def test_pid_value_none_returns_false(self):
        """Test pid_value None returns False"""
        self.assertFalse(
            pid_utils.is_valid_pid_value(
                None, self.mock_pid_provider_name, self.mock_pid_format
            )
        )

    def test_pid_value_empty_string_returns_false(self):
        """Test pid_value empty returns False"""
        self.assertFalse(
            pid_utils.is_valid_pid_value(
                "", self.mock_pid_provider_name, self.mock_pid_format
            )
        )

    @patch("core_linked_records_app.utils.providers.ProviderManager.get")
    def test_pid_value_not_match_returns_false(
        self, mock_provider_manager_get
    ):
        """Test pid_value not matching returns False"""
        mock_provider_manager_get.return_value = self.mock_provider

        self.assertFalse(
            pid_utils.is_valid_pid_value(
                "wrong_pid", self.mock_pid_provider_name, self.mock_pid_format
            )
        )

    @patch("core_linked_records_app.utils.providers.ProviderManager.get")
    def test_pid_value_match_returns_true(self, mock_provider_manager_get):
        """Test pid_value matching returns True"""
        mock_provider_manager_get.return_value = self.mock_provider

        mock_pid = f"{self.mock_provider_url}/{test_settings.ID_PROVIDER_PREFIXES[0]}/mockpid"

        self.assertTrue(
            pid_utils.is_valid_pid_value(
                mock_pid, self.mock_pid_provider_name, self.mock_pid_format
            )
        )


class TestGetPidSettingsDict(TestCase):
    """Unit tests for `get_pid_settings_dict` function."""

    def test_returns_dict(self):
        """test_returns_dict"""
        result = pid_utils.get_pid_settings_dict(MagicMock())
        self.assertIsInstance(result, dict)

    def test_returns_expected_keys(self):
        """test_returns_expected_keys"""
        expected_keys = [
            "auto_set_pid",
            "path",
            "format",
            "system_name",
            "system_type",
            "prefixes",
        ]

        result = pid_utils.get_pid_settings_dict(MagicMock())
        self.assertEqual(sorted(result.keys()), sorted(expected_keys))


class TestSplitPrefixFromRecord(TestCase):
    """Unit tests for `split_prefix_from_record` function."""

    @patch.object(pid_utils, "settings")
    def test_no_record_provided_with_no_final_slash_sets_values_correctly(
        self, mock_settings
    ):
        """test_no_record_provided_with_no_final_slash_sets_values_correctly"""
        mock_prefix = "mock_prefix"
        mock_settings.ID_PROVIDER_PREFIXES = [mock_prefix]

        expected_results = (mock_prefix, None)
        results = pid_utils.split_prefix_from_record(mock_prefix)

        self.assertEqual(results, expected_results)

    @patch.object(pid_utils, "settings")
    def test_no_record_provided_with_final_slash_sets_values_correctly(
        self, mock_settings
    ):
        """test_no_record_provided_with_final_slash_sets_values_correctly"""
        mock_prefix = "mock_prefix/"
        mock_settings.ID_PROVIDER_PREFIXES = [mock_prefix[:-1]]

        expected_results = (mock_prefix[:-1], None)
        results = pid_utils.split_prefix_from_record(mock_prefix)

        self.assertEqual(results, expected_results)

    @patch.object(pid_utils, "settings")
    def test_prefix_and_record_provided_sets_values_correctly(
        self, mock_settings
    ):
        """test_prefix_and_record_provided_sets_values_correctly"""
        mock_prefix = "mock_prefix"
        mock_record = "mock_record"
        mock_settings.ID_PROVIDER_PREFIXES = [mock_prefix]
        mock_settings.PID_FORMAT = r"[a-z_]+"

        expected_results = (mock_prefix, mock_record)
        results = pid_utils.split_prefix_from_record(
            f"{mock_prefix}/{mock_record}"
        )

        self.assertEqual(results, expected_results)

    def test_empty_prefix_raises_invalid_prefix_error(self):
        """test_empty_prefix_raises_invalid_prefix_error"""
        mock_prefix = ""
        mock_record = "mock_record"

        with self.assertRaises(InvalidPrefixError):
            pid_utils.split_prefix_from_record(f"{mock_prefix}/{mock_record}")

    @patch.object(pid_utils, "settings")
    def test_invalid_prefix_raises_invalid_prefix_error(self, mock_settings):
        """test_invalid_prefix_raises_invalid_prefix_error"""
        mock_prefix = "mock_prefix"
        mock_record = "mock_record"
        mock_settings.ID_PROVIDER_PREFIXES = [mock_prefix + "_0"]

        with self.assertRaises(InvalidPrefixError):
            pid_utils.split_prefix_from_record(f"{mock_prefix}/{mock_record}")

    @patch.object(pid_utils, "settings")
    def test_invalid_record_raises_invalid_record_error(self, mock_settings):
        """test_invalid_record_raises_invalid_record_error"""
        mock_prefix = "mock_prefix"
        mock_record = "mock_record"
        mock_settings.ID_PROVIDER_PREFIXES = [mock_prefix]

        with self.assertRaises(InvalidRecordError):
            pid_utils.split_prefix_from_record(f"{mock_prefix}/{mock_record}")



""" Custom admin site for the PidSettings model
"""
from django.contrib import admin


class CustomPidSettingsAdmin(admin.ModelAdmin):
    """CustomPidSettingsAdmin"""

    def has_add_permission(
        self, request, obj=None
    ):  # pylint: disable=unused-argument
        """Prevent from manually adding PidSettings"""
        return False

    def has_delete_permission(self, request, obj=None):
        """Prevent from manually deleting PidSettings"""
        return False



"""  Admin
"""

from django.contrib import admin
from django.contrib.admin.views.decorators import staff_member_required
from django.urls import re_path

from core_linked_records_app.components.local_id.admin_site import (
    CustomLocalIdAdmin,
)
from core_linked_records_app.components.local_id.models import LocalId
from core_linked_records_app.components.pid_settings.admin_site import (
    CustomPidSettingsAdmin,
)
from core_linked_records_app.components.pid_settings.models import PidSettings
from core_linked_records_app.components.pid_path.admin_site import (
    CustomPidPathAdmin,
)
from core_linked_records_app.components.pid_path.models import PidPath
from core_linked_records_app.views.admin import views as admin_views
from core_main_app.admin import core_admin_site

admin_urls = [
    re_path(
        r"^pid/settings/$",
        staff_member_required(admin_views.PidSettingsView.as_view()),
        name="core_linked_records_app_admin_settings",
    ),
]

admin.site.register(LocalId, CustomLocalIdAdmin)
admin.site.register(PidSettings, CustomPidSettingsAdmin)
admin.site.register(PidPath, CustomPidPathAdmin)

urls = core_admin_site.get_urls()
core_admin_site.get_urls = lambda: admin_urls + urls



""" Unit tests for core_linked_records_app.utils.providers.handle_net
"""
import json
from unittest import TestCase
from unittest.mock import patch

from core_linked_records_app.utils.providers.handle_net import HandleNetSystem
from tests.mocks import MockResponse


class TestHandleNetSystemUpdateResponseContent(TestCase):
    """Test Handle Net System Update Response Content"""

    def test_returns_dict_with_message_key(self):
        """test_returns_dict_with_message_key"""

        pass

    def test_returns_dict_with_url_key(self):
        """test_returns_dict_with_url_key"""

        pass


class TestHandleNetSystemEncodeToken(TestCase):
    """Test Handle Net System Encode Token"""

    def test_returned_base64_encoding(self):
        """test_returned_base64_encoding"""

        pass


class TestHandleNetSystemGet(TestCase):
    """Test Handle Net System Get"""

    def test_record_url_called(self):
        """test_record_url_called"""

        pass

    def test_returns_dict_with_message_key(self):
        """test_returns_dict_with_message_key"""

        pass

    def test_returns_dict_with_url_key(self):
        """test_returns_dict_with_url_key"""

        pass


class TestHandleNetSystemCreate(TestCase):
    """Test Handle Net System Create"""

    def test_record_not_none_calls_existing_record_url(self):
        """test_record_not_none_calls_existing_record_url"""

        pass

    def test_record_not_none_returns_dict_with_message_key(self):
        """test_record_not_none_returns_dict_with_message_key"""

        pass

    def test_record_not_none_returns_dict_with_url_key(self):
        """test_record_not_none_returns_dict_with_url_key"""
        pass

    def test_record_none_calls_new_record_url(self):
        """test_record_none_calls_new_record_url"""

        pass

    def test_record_none_calls_update_method(self):
        """test_record_none_calls_update_method"""
        pass


class TestHandleNetSystemUpdate(TestCase):
    """Test Handle Net System Update"""

    def setUp(self) -> None:
        self.mock_handle_system = HandleNetSystem(
            "mock_provider_name",
            "mock_provider_lookup_url",
            "mock_provider_registration_url",
            "mock_username",
            "mock_password",
        )

    @patch(
        "core_linked_records_app.utils.providers.handle_net.send_put_request"
    )
    def test_send_put_request_on_record_url(self, mock_send_put_request):
        """test_send_put_request_on_record_url"""
        mock_record = "mock_record"
        mock_put_response = {"handle": "mock_handle", "responseCode": 000}
        mock_send_put_request.return_value = MockResponse(
            content=json.dumps(mock_put_response)
        )

        self.mock_handle_system.update(mock_record)
        mock_send_put_request.assert_called_with(
            f"{self.mock_handle_system.provider_registration_url}/"
            f"{self.mock_handle_system.registration_api}/{mock_record}?overwrite=true",
            self.mock_handle_system._generate_record_data(
                mock_record, include_handle=True
            ),
            headers={
                "Content-Type": "application/json",
                "Authorization": f"Basic {str(self.mock_handle_system.auth_token)}",
            },
        )

    def test_returns_dict_with_message_key(self):
        """test_returns_dict_with_message_key"""

        pass

    def test_returns_dict_with_url_key(self):
        """test_returns_dict_with_url_key"""

        pass


class TestHandleNetSystemDelete(TestCase):
    """Test Handle Net System Delete"""

    def test_send_delete_request_on_record_url(self):
        """test_send_delete_request_on_record_url"""

        pass

    def test_returns_dict_with_message_key(self):
        """test_returns_dict_with_message_key"""

        pass

    def test_returns_dict_with_url_key(self):
        """test_returns_dict_with_url_key"""

        pass



""" API for PidPath model
"""
import logging

from core_linked_records_app import settings
from core_linked_records_app.components.pid_path.access_control import (
    can_get_by_template,
)
from core_linked_records_app.components.pid_path.models import PidPath
from core_main_app.access_control.decorators import access_control
from core_main_app.commons.exceptions import ApiError
from core_main_app.components.template import (
    api as template_api,
)

logger = logging.getLogger(__name__)


@access_control(can_get_by_template)
def get_by_template(template, user):  # noqa, pylint: disable=unused-argument
    """Retrieve PID path associated with a specific template ID

    Args:
        template: Template
        user: User

    Returns:
        str - Dot notation path for the given template ID.
    """
    try:
        pid_path_object = PidPath.get_by_template(template)

        # Returns default PID_PATH settings if the template has no defined PidPath
        if pid_path_object is None:
            return PidPath(
                template=template,
                path=settings.PID_PATH,
            )

        return pid_path_object
    except Exception as exc:
        error_message = (
            f"An unexpected error occurred while retrieving PidPath "
            f"assigned to template {template.pk}"
        )

        logger.error("%s: %s", error_message, str(exc))
        raise ApiError(str(exc)) from exc


def get_all(request):
    """Retrieve all PID paths stored in `PidSettings`.

    Args:
        request (HttpRequest): The HTTP Request.

    Returns:
        list<str>: List of PID path.
    """
    try:
        return PidPath.get_all_by_template_list(
            [template.pk for template in template_api.get_all(request=request)]
        )
    except Exception as exc:
        error_message = (
            "An unexpected error occurred while retrieving all PidPath"
        )

        logger.error("%s: %s", error_message, str(exc))
        raise ApiError(str(exc)) from exc



""" Settings for core_linked_records_app package.

Settings with the following syntax can be overwritten at the project level:
SETTING_NAME = getattr(settings, "SETTING_NAME", "Default Value")
"""
from django.conf import settings

from core_main_app.utils.settings import getattr_from_deprecated_var

if not settings.configured:
    settings.configure()

INSTALLED_APPS = getattr(
    settings,
    "INSTALLED_APPS",
    [],
)
SERVER_URI = getattr(settings, "SERVER_URI", "http://localhost")

CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT = getattr(
    settings, "CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT", False
)

PID_PATH = getattr_from_deprecated_var(
    settings, "PID_XPATH", "PID_PATH", "Resource.@localid"
)

PID_FORMAT = getattr(settings, "PID_FORMAT", r"[a-zA-Z0-9_\-]+")

ID_PROVIDER_SYSTEM_NAME = getattr(settings, "ID_PROVIDER_SYSTEM_NAME", "local")

ID_PROVIDER_SYSTEM_CONFIG = getattr(
    settings,
    "ID_PROVIDER_SYSTEM_CONFIG",
    {
        "class": "core_linked_records_app.utils.providers.local.LocalIdProvider",
        "args": [],
    },
)

ID_PROVIDER_PREFIXES = getattr(settings, "ID_PROVIDER_PREFIXES", ["cdcs"])

ID_PROVIDER_PREFIX_DEFAULT = getattr(
    settings, "ID_PROVIDER_PREFIX_DEFAULT", ID_PROVIDER_PREFIXES[0]
)

ID_PROVIDER_PREFIX_BLOB = getattr(
    settings, "ID_PROVIDER_PREFIX_BLOB", ID_PROVIDER_PREFIXES[0]
)

HANDLE_NET_RECORD_INDEX = getattr(settings, "HANDLE_NET_RECORD_INDEX", 1)

HANDLE_NET_ADMIN_DATA = getattr(
    settings,
    "HANDLE_NET_ADMIN_DATA",
    {
        "index": 100,
        "type": "HS_ADMIN",
        "data": {
            "format": "admin",
            "value": {
                "handle": f"0.NA/{ID_PROVIDER_PREFIX_DEFAULT}",
                "index": 200,
                "permissions": "011111110011",
            },
        },
    },
)

AUTO_SET_PID = getattr(settings, "AUTO_SET_PID", False)

BACKWARD_COMPATIBILITY_DATA_XML_CONTENT = getattr(
    settings, "BACKWARD_COMPATIBILITY_DATA_XML_CONTENT", True
)



""" Url router for the explore OAI-PMH search application
"""
from django.conf.urls import include
from django.contrib import admin
from django.urls import re_path

from core_main_app.admin import core_admin_site

admin.autodiscover()

urlpatterns = [
    re_path(r"^admin/", admin.site.urls),
    re_path(r"^core-admin/", core_admin_site.urls),
    re_path(r"^", include("core_main_app.urls")),
    re_path(r"^", include("core_linked_records_app.urls")),
]



""" Utilities related to PID
"""
import re

from core_linked_records_app import settings
from core_linked_records_app.utils.exceptions import (
    InvalidPrefixError,
    InvalidRecordError,
)
from core_linked_records_app.utils.providers import ProviderManager


def is_valid_pid_value(pid_value, pid_provider_name, pid_format):
    """Check if a provided PID has a valid URL according to the provided settings

    Args:
        pid_value: str - Value of the PID
        pid_provider_name: str - Expected provider of the PID
        pid_format: str - Regexp format of the record

    Returns:
        bool - True if the PID matches the regexp, False otherwise.
    """
    if not pid_value:  # Don't test if pid_value is None or ''
        return False

    # Retrieve the active provider
    provider = ProviderManager().get(pid_provider_name)

    # Build regexp and test if it matches the pid_value
    pid_prefixes_regexp = "|".join(settings.ID_PROVIDER_PREFIXES)
    pid_regexp_match = f"{provider.provider_lookup_url}/(?:{pid_prefixes_regexp})/{pid_format}"

    return re.match(pid_regexp_match, pid_value) is not None


def get_pid_settings_dict(pid_setting) -> dict:
    """Retrieve all settings related to PID configuration and returns a dictionary.

    Args:
        pid_setting: PidSettings object from DB.

    Returns:
         Dictionary containing all PID settings in DB or configuration files.
    """
    return {
        "auto_set_pid": pid_setting.auto_set_pid,
        "path": settings.PID_PATH,
        "format": settings.PID_FORMAT,
        "system_name": settings.ID_PROVIDER_SYSTEM_NAME,
        "system_type": settings.ID_PROVIDER_SYSTEM_CONFIG["class"],
        "prefixes": settings.ID_PROVIDER_PREFIXES,
    }


def split_prefix_from_record(record):
    """Split prefix from record if the record is in prefix/record format.

    Args:
        record (str): The full prefix/record_name to be split

    Raises:
        InvalidPrefixError: If an undefined prefix is detected.
        InvalidRecordError: If the record does not match the expected format.

    Returns:
        tuple[str, str|None] - prefix / record
    """
    prefix_record_list = record.split("/")

    if len(prefix_record_list) == 1:  # No record provided
        prefix = prefix_record_list[0]
        record = None
    elif prefix_record_list[-1] == "":  # Only prefix provided
        prefix = "/".join(prefix_record_list[:-1])
        record = None
    else:  # Prefix and record provided
        prefix = "/".join(prefix_record_list[:-1])
        record = prefix_record_list[-1]

    # Assign default prefix if the prefix is undefined or not in the
    # list of authorized ones.
    if prefix == "" or prefix not in settings.ID_PROVIDER_PREFIXES:
        raise InvalidPrefixError(f"Prefix {prefix} is not valid.")

    if (
        record is not None
        and record != ""
        and re.match(f"^({settings.PID_FORMAT}|)$", record) is None
    ):
        raise InvalidRecordError(f"Record {record} is not valid.")

    return prefix, record



""" Signals to trigger before Data save
"""
import json
import logging

from django.db import transaction
from django.db.models.signals import pre_save, post_delete
from django.urls import resolve
from rest_framework import status

from core_linked_records_app import settings
from core_linked_records_app.components.pid_settings.models import PidSettings
from core_linked_records_app.system.data import api as data_system_api
from core_linked_records_app.system.pid_path import (
    api as pid_path_system_api,
)
from core_linked_records_app.utils import data as data_utils
from core_linked_records_app.utils import exceptions
from core_linked_records_app.utils.pid import split_prefix_from_record
from core_linked_records_app.utils.providers import (
    ProviderManager,
    retrieve_provider_name,
)
from core_main_app.components.data.models import Data

logger = logging.getLogger(__name__)


def init():
    """Connect to Data object events."""
    pre_save.connect(set_data_pid, sender=Data)
    post_delete.connect(delete_data_pid, sender=Data)


def _register_pid_for_data_id(provider_name, pid_value, data_id):
    """Registers a data with a set PID to a given provider.

    Args:
        provider_name:
        pid_value:
        data_id:

    Returns:
        str - Persistent identifier
    """
    # Retrieve provider, prefix and record information to register the record in
    # the default provider.
    try:
        # Retrieve the current provider.
        provider_manager = ProviderManager()
        provider = provider_manager.get(provider_name)

        # Parse the `pid_value` to obtain record & prefix info.
        local_pid_url = pid_value.replace(
            provider.provider_lookup_url, provider.local_url
        )

        # Try resolving the `local_pid_url` and ensure the expected view was
        # retrieved.
        resolver_match = resolve(
            local_pid_url.replace(settings.SERVER_URI, "")
        )
        assert (
            resolver_match.view_name == "core_linked_records_provider_record"
        )

        # Before asking the provider to create a record, separate prefix from
        # record name.
        prefix, record = split_prefix_from_record(
            resolver_match.kwargs["record"]
        )
    except Exception as exc:
        error_message = (
            "An unexpected error occurred while retrieving information to "
            "register the PID"
        )
        logger.error("%s: %s", error_message, str(exc))
        raise exceptions.PidResolverError(f"{error_message}.")

    provider_response = provider.create(prefix, record)

    # If an error happened during PID registration, try to relay the message
    # from the provider, otherwise relay a default error message.
    if (
        provider_response.status_code != status.HTTP_201_CREATED
        and provider_response.status_code != status.HTTP_200_OK
        and data_system_api.get_data_by_pid(pid_value).pk != data_id
    ):
        default_error_message = "An error occurred while creating the PID"
        try:
            error_message = json.loads(provider_response.content).get(
                "message", default_error_message
            )
            logger.error(error_message)
            raise exceptions.PidCreateError(error_message)
        except Exception as exc:  # If the response is not JSON parsable
            logger.error("%s: %s", default_error_message, str(exc))
            raise exceptions.PidCreateError(default_error_message)

    return json.loads(provider_response.content)["url"]


def _set_data_pid(instance: Data):
    """Set the PID in the field specified in the settings. If the PID
    already exists and is valid, it is not reset.

    Args:
        instance:

    Returns:
    """
    try:
        if not PidSettings.get().auto_set_pid:
            return

        # Retrieve PID path from `PidSettings.path_list`. Skip PID assignment
        # if the PID path is not defined for the template.
        template_pid_path = pid_path_system_api.get_pid_path_by_template(
            instance.template
        )
        pid_path = template_pid_path.path

        try:  # Retrieve the PID located at predefined dot notation path.
            pid_value = data_utils.get_pid_value_for_data(instance, pid_path)
        except Exception as exc:  # pylint: disable=broad-except
            # PID path is not valid for current instance.
            logger.warning(
                "Cannot create PID at %s for data %s: %s",
                pid_path,
                instance.pk,
                str(exc),
            )
            return

        # Remove previous instance PID from DB.
        if instance.pk is not None:
            transaction.on_commit(
                lambda: data_system_api.delete_pid_for_data(instance)
            )

        provider_name = retrieve_provider_name(pid_value)

        # Assign default value for undefined PID and check that the PID is not
        # defined for a instance other than the current instance.
        if pid_value is None or pid_value == "":
            pid_value = (
                f"{ProviderManager().get(provider_name).provider_lookup_url}/"
                f"{settings.ID_PROVIDER_PREFIX_DEFAULT}"
            )

        if data_system_api.is_pid_defined(pid_value) and (
            instance.pk is None
            or not data_system_api.is_pid_defined_for_data(
                pid_value, instance.pk
            )
        ):
            raise exceptions.PidCreateError(
                "PID already defined for another instance"
            )

        # Register PID and write resulting URL in instance
        pid_value = _register_pid_for_data_id(
            provider_name, pid_value, instance.pk
        )
        data_utils.set_pid_value_for_data(instance, pid_path, pid_value)
    except exceptions.PidCreateError as pid_create_error:
        logger.error(
            "An error occurred while assigning PID: %s", str(pid_create_error)
        )
        raise exceptions.PidCreateError(str(pid_create_error))
    except Exception as exc:
        if not instance.pk:
            data_definition = "new data"
        else:
            data_definition = f"data '{instance.pk}'"

        error_message = (
            f"An error occurred while assigning PID to {data_definition}"
        )

        logger.error("%s: %s", error_message, str(exc))
        raise exceptions.PidCreateError(f"{error_message}.")


def set_data_pid(
    sender, instance: Data, **kwargs  # noqa, pylint: disable=unused-argument
):
    """Wrapper around `_set_data_pid` to ensure the PID can be updated while
    the Data is being saved without locking the DB.

    Args:
        sender:
        instance:
        kwargs:

    Returns:
    """
    transaction.on_commit(lambda: _set_data_pid(instance))


def delete_data_pid(
    sender, instance: Data, **kwargs  # noqa, pylint: disable=unused-argument
):
    """Delete a PID assigned to a Data

    Args:
        sender:
        instance:
        kwargs:
    """
    try:
        data_system_api.delete_pid_for_data(instance)
    except Exception as exc:  # pylint: disable=broad-except
        logger.warning(
            "Trying to delete PID for data %d but an error occurred: %s",
            instance.pk,
            str(exc),
        )



""" Access control methods for `core_linked_records.components.oai_record.api`.
"""
from core_main_app.access_control.api import check_anonymous_access


def can_get_pid_for_data(func, oai_record_id, request):
    """Access control for the `get_pid_for_data` function.

    Args:
        func:
        oai_record_id:
        request:

    Returns:
    """
    check_anonymous_access(request.user)
    return func(oai_record_id, request)



""" Unit tests for core_linked_records_app.rest.query.views
"""
from unittest import TestCase
from unittest.mock import patch, Mock

from core_linked_records_app.components.pid_path import api as pid_path_api
from core_linked_records_app.utils import query as query_utils
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons.exceptions import ApiError, CoreError
from core_main_app.components.data import api as data_api
from core_oaipmh_harvester_app.components.oai_record import (
    api as oai_record_api,
)
from tests import mocks


class TestBuildPidQuery(TestCase):
    """Test Execute Local PID Query View Build Query"""

    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest()

    @patch.object(pid_path_api, "get_all")
    def test_add_pid_query_if_init_query_has_and(self, mock_pid_path_get_all):
        """test_add_pid_query_if_init_query_has_and"""

        mock_pid_path_get_all.return_value = []
        mock_function = Mock()
        mock_function.return_value = {"mock_key": "mock_value"}

        result = query_utils.build_pid_query(
            "mock_query", mock_function, self.mock_request
        )

        self.assertEqual(len(result["$and"]), 2)

    @patch.object(pid_path_api, "get_all")
    def test_append_pid_query_if_init_query_has_no_and(
        self, mock_pid_path_get_all
    ):
        """test_append_pid_query_if_init_query_has_no_and"""

        mock_pid_path_get_all.return_value = []
        mock_function = Mock()
        mock_function.return_value = {"$and": ["query1", "query2"]}

        result = query_utils.build_pid_query(
            "mock_query", mock_function, self.mock_request
        )

        self.assertEqual(len(result["$and"]), 3)


class TestExecutePidQuery(TestCase):
    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest()

    @patch.object(query_utils, "build_pid_query")
    def test_build_pid_query_unauthorized_raises_acl_error(
        self, mock_build_pid_query
    ):
        mock_build_pid_query.side_effect = AccessControlError(
            "mock_build_pid_query_exception"
        )

        with self.assertRaises(AccessControlError):
            query_utils.execute_pid_query(
                "mock_query", Mock(), Mock(), self.mock_request
            )

    @patch.object(query_utils, "build_pid_query")
    def test_execute_fn_unauthorized_raises_acl_error(
        self, mock_build_pid_query
    ):
        mock_build_pid_query.return_value = "mock_pid_query"
        mock_execute_fn = Mock()
        mock_execute_fn.side_effect = AccessControlError(
            "mock_execute_fn_exception"
        )

        with self.assertRaises(AccessControlError):
            query_utils.execute_pid_query(
                "mock_query", Mock(), mock_execute_fn, self.mock_request
            )

    @patch.object(query_utils, "build_pid_query")
    def test_build_pid_query_fail_raises_api_error(self, mock_build_pid_query):
        mock_build_pid_query.side_effect = Exception(
            "mock_build_pid_query_exception"
        )

        with self.assertRaises(ApiError):
            query_utils.execute_pid_query(
                "mock_query", Mock(), Mock(), self.mock_request
            )

    @patch.object(query_utils, "build_pid_query")
    def test_execute_fn_fail_raises_api_error(self, mock_build_pid_query):
        mock_build_pid_query.return_value = "mock_pid_query"
        mock_execute_fn = Mock()
        mock_execute_fn.side_effect = Exception("mock_execute_fn_exception")

        with self.assertRaises(ApiError):
            query_utils.execute_pid_query(
                "mock_query", Mock(), mock_execute_fn, self.mock_request
            )

    @patch.object(query_utils, "build_pid_query")
    def test_success_returns_execute_fn(self, mock_build_pid_query):
        mock_build_pid_query.return_value = "mock_pid_query"
        expected_results = "mock_results"
        mock_execute_fn = Mock()
        mock_execute_fn.return_value = expected_results

        results = query_utils.execute_pid_query(
            "mock_query", Mock(), mock_execute_fn, self.mock_request
        )
        self.assertEqual(results, expected_results)


class TestExecuteLocalQuery(TestCase):
    """Test Execute Local PID Query View Execute Raw Query"""

    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest()

    @patch.object(data_api, "execute_json_query")
    def test_no_data_returns_empty_list(self, mock_execute_query):
        """test_no_data_returns_empty_list"""

        mock_execute_query.return_value = []

        result = query_utils.execute_local_query(
            "mock_query", self.mock_request
        )

        self.assertEqual(result, [])

    @patch.object(query_utils, "is_valid_pid_value")
    @patch.object(query_utils, "get_value_from_dot_notation")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(data_api, "execute_json_query")
    def test_returns_data_with_valid_pid(
        self,
        mock_execute_query,
        mock_get_by_template,
        mock_get_value_from_dot_notation,
        mock_is_valid_pid_value,
    ):
        """test_returns_data_with_valid_pid"""

        mock_data_pid = "mock_data_pid"
        mock_execute_query.return_value = [mocks.MockData() for _ in range(5)]
        mock_get_by_template.return_value = mocks.MockPidPath()
        mock_get_value_from_dot_notation.return_value = mock_data_pid
        # Return True every time the call count is odd (3 times for a list of 5
        # elements, at index 0, 2 and 4).
        mock_is_valid_pid_value.side_effect = (
            lambda p, n, f: mock_is_valid_pid_value.call_count % 2
        )
        expected_result = [mock_data_pid for _ in range(5) if _ % 2 == 0]

        result = query_utils.execute_local_query(
            "mock_query", self.mock_request
        )

        self.assertEqual(result, expected_result)


class TestExecuteLocalPidQuery(TestCase):
    @patch.object(query_utils, "execute_pid_query")
    def test_returns_execute_pid_query_result(self, mock_execute_pid_query):
        expected_result = "mock_result"
        mock_execute_pid_query.return_value = expected_result

        result = query_utils.execute_local_pid_query(
            "mock_query", mocks.MockRequest()
        )
        self.assertEqual(result, expected_result)


class TestExecuteOaiPmhQuery(TestCase):
    """Test Execute Local PID Query View Execute Raw Query"""

    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest()

    @patch.object(query_utils, "settings")
    def test_no_oaipmh_harvester_raises_core_error(self, mock_settings):
        mock_settings.INSTALLED_APPS = ["core_explore_oaipmh_app"]

        with self.assertRaises(CoreError):
            query_utils.execute_oaipmh_query("mock_query", self.mock_request)

    @patch.object(oai_record_api, "execute_json_query")
    def test_no_data_returns_empty_list(self, mock_execute_query):
        """test_no_data_returns_empty_list"""

        mock_execute_query.return_value = []

        result = query_utils.execute_oaipmh_query(
            "mock_query", self.mock_request
        )

        self.assertEqual(result, [])

    @patch.object(query_utils, "get_value_from_dot_notation")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(oai_record_api, "execute_json_query")
    def test_returns_data_with_valid_pid(
        self,
        mock_execute_query,
        mock_get_by_template,
        mock_get_value_from_dot_notation,
    ):
        """test_returns_data_with_valid_pid"""

        mock_data_pid = "mock_data_pid"
        mock_execute_query.return_value = [mocks.MockData() for _ in range(5)]
        mock_get_by_template.return_value = mocks.MockPidPath()
        # Return `mock_data_pid` every time the call count is odd (3 times for a list
        # of 5 elements, at index 0, 2 and 4), otherwise returns None.
        mock_get_value_from_dot_notation.side_effect = (
            lambda d, p: mock_data_pid
            if mock_get_value_from_dot_notation.call_count % 2
            else None
        )
        expected_result = [mock_data_pid for _ in range(3)]

        result = query_utils.execute_oaipmh_query(
            "mock_query", self.mock_request
        )

        self.assertEqual(result, expected_result)


class TestExecuteOaiPmhPidQuery(TestCase):
    @patch.object(query_utils, "execute_pid_query")
    @patch.object(query_utils, "settings")
    def test_no_explore_oaipmh_raises_core_error(
        self, mock_settings, mock_execute_pid_query
    ):
        expected_result = "mock_result"
        mock_execute_pid_query.return_value = expected_result
        mock_settings.INSTALLED_APPS = []

        with self.assertRaises(CoreError):
            query_utils.execute_oaipmh_pid_query(
                "mock_query", mocks.MockRequest()
            )

    @patch.object(query_utils, "execute_pid_query")
    def test_returns_execute_pid_query_result(self, mock_execute_pid_query):
        expected_result = "mock_result"
        mock_execute_pid_query.return_value = expected_result

        result = query_utils.execute_oaipmh_pid_query(
            "mock_query", mocks.MockRequest()
        )
        self.assertEqual(result, expected_result)



""" Unit tests for `core_linked_records.utils.json` package.
"""
from unittest import TestCase
from unittest.mock import MagicMock, patch

from core_linked_records_app.utils import json as json_utils


class TestSetValueAtDictPath(TestCase):
    """Unit tests for `set_value_at_dict_path` function."""

    def setUp(self):
        """setUp"""
        self.mock_kwargs = {
            "dictionary": MagicMock(),
            "dot_notation": MagicMock(),
            "value": MagicMock(),
        }

    def test_no_key_raises_key_error(self):
        """test_no_key_raises_key_error"""
        self.mock_kwargs["dot_notation"] = None  # noqa

        with self.assertRaises(KeyError):
            json_utils.set_value_at_dict_path(**self.mock_kwargs)

    def test_single_key_returns_modified_dictionary(self):
        """test_single_key_returns_modified_dictionary"""
        self.mock_kwargs["dictionary"] = {"key1": MagicMock()}  # noqa
        self.mock_kwargs["dot_notation"] = "key1"  # noqa

        self.assertEqual(
            json_utils.set_value_at_dict_path(**self.mock_kwargs),
            {"key1": self.mock_kwargs["value"]},
        )

    def test_multiple_key_returns_modified_dictionary(self):
        """test_multiple_key_returns_modified_dictionary"""
        self.mock_kwargs["dictionary"] = {  # noqa
            "key1": {"key2": MagicMock()}
        }
        self.mock_kwargs["dot_notation"] = "key1.key2"  # noqa

        self.assertEqual(
            json_utils.set_value_at_dict_path(**self.mock_kwargs),
            {"key1": {"key2": self.mock_kwargs["value"]}},
        )


class TestCanCreateValueAtDictPath(TestCase):
    """Unit tests for `can_create_value_at_dict_path` function."""

    def setUp(self):
        """setUp"""
        self.mock_kwargs = {
            "json_dict": MagicMock(),
            "template_dict": MagicMock(),
            "dict_path": MagicMock(),
            "value": MagicMock(),
        }

    @patch.object(json_utils, "validate_json_data")
    @patch.object(json_utils, "set_value_at_dict_path")
    def test_set_value_at_dict_path_called(
        self,
        mock_set_value_at_dict_path,
        mock_validate_json_data,  # noqa, pylint: disable=unused-argument
    ):
        """test_set_value_at_dict_path_called"""
        json_utils.can_create_value_at_dict_path(**self.mock_kwargs)

        mock_set_value_at_dict_path.assert_called_with(
            self.mock_kwargs["json_dict"],
            self.mock_kwargs["dict_path"],
            self.mock_kwargs["value"],
        )

    @patch.object(json_utils, "validate_json_data")
    @patch.object(json_utils, "set_value_at_dict_path")
    def test_validate_json_data_called(
        self,
        mock_set_value_at_dict_path,  # noqa, pylint: disable=unused-argument
        mock_validate_json_data,
    ):
        """test_validate_json_data_called"""
        json_utils.can_create_value_at_dict_path(**self.mock_kwargs)

        mock_validate_json_data.assert_called_with(
            self.mock_kwargs["json_dict"], self.mock_kwargs["template_dict"]
        )

    @patch.object(json_utils, "validate_json_data")
    @patch.object(json_utils, "set_value_at_dict_path")
    def test_validate_json_data_not_none_returns_false(
        self,
        mock_set_value_at_dict_path,  # noqa, pylint: disable=unused-argument
        mock_validate_json_data,
    ):
        """test_validate_json_data_not_none_returns_false"""
        mock_validate_json_data.return_value = "mock_validation_error"
        self.assertFalse(
            json_utils.can_create_value_at_dict_path(**self.mock_kwargs)
        )

    @patch.object(json_utils, "validate_json_data")
    @patch.object(json_utils, "set_value_at_dict_path")
    def test_successful_execution_returns_true(
        self,
        mock_set_value_at_dict_path,  # noqa, pylint: disable=unused-argument
        mock_validate_json_data,
    ):
        """test_successful_execution_returns_true"""
        mock_validate_json_data.return_value = None
        self.assertTrue(
            json_utils.can_create_value_at_dict_path(**self.mock_kwargs)
        )



""" Unit tests for core_linked_records_app.utils.blob
"""
from unittest import TestCase
from unittest.mock import patch, MagicMock

from core_linked_records_app.utils import blob as blob_utils


class TestGetBlobDownloadRegex(TestCase):
    """Test Get Blob Download Regex"""

    @patch.object(blob_utils, "blob_system_api")
    @patch.object(blob_utils, "local_id_system_api")
    @patch.object(blob_utils, "re")
    @patch.object(blob_utils, "reverse")
    def test_reverse_called(
        self,
        mock_reverse,
        mock_regex_module,  # noqa, pylint: disable=unused-argument
        mock_local_id_system_api,  # noqa, pylint: disable=unused-argument
        mock_blob_system_api,  # noqa, pylint: disable=unused-argument
    ):
        """test_reverse_called"""
        blob_utils.get_blob_download_regex("mock_xml_string")

        mock_reverse.assert_called_with(
            "core_linked_records_provider_record",
            kwargs={"provider": "mock_string", "record": "mock_string"},
        )

    @patch.object(blob_utils, "blob_system_api")
    @patch.object(blob_utils, "local_id_system_api")
    @patch.object(blob_utils, "re")
    @patch.object(blob_utils, "reverse")
    def test_local_id_get_by_name_called(
        self,
        mock_reverse,  # noqa, pylint: disable=unused-argument
        mock_regex_module,
        mock_local_id_system_api,
        mock_blob_system_api,  # noqa, pylint: disable=unused-argument
    ):
        """test_local_id_get_by_name_called"""
        mock_regex_module.findall.return_value = [MagicMock()]

        blob_utils.get_blob_download_regex("mock_xml_string")

        mock_local_id_system_api.get_by_name.assert_called()

    @patch.object(blob_utils, "blob_system_api")
    @patch.object(blob_utils, "local_id_system_api")
    @patch.object(blob_utils, "re")
    @patch.object(blob_utils, "reverse")
    @patch.object(blob_utils, "logger")
    def test_local_id_get_by_name_error_is_logged(
        self,
        mock_logger,
        mock_reverse,  # noqa, pylint: disable=unused-argument
        mock_regex_module,
        mock_local_id_system_api,
        mock_blob_system_api,  # noqa, pylint: disable=unused-argument
    ):
        """test_local_id_get_by_name_error_is_logged"""
        mock_regex_module.findall.return_value = [MagicMock()]
        mock_local_id_system_api.get_by_name.side_effect = Exception(
            "mock_get_by_name_exception"
        )

        blob_utils.get_blob_download_regex("mock_xml_string")
        mock_logger.warning.assert_called()

    @patch.object(blob_utils, "blob_system_api")
    @patch.object(blob_utils, "local_id_system_api")
    @patch.object(blob_utils, "re")
    @patch.object(blob_utils, "reverse")
    def test_get_pid_for_blob_called(
        self,
        mock_reverse,  # noqa, pylint: disable=unused-argument
        mock_regex_module,
        mock_local_id_system_api,
        mock_blob_system_api,
    ):
        """test_get_pid_for_blob_called"""
        mock_record_object = MagicMock()
        mock_regex_module.findall.return_value = [MagicMock()]
        mock_local_id_system_api.get_by_name.return_value = mock_record_object

        blob_utils.get_blob_download_regex("mock_xml_string")
        mock_blob_system_api.get_pid_for_blob.assert_called_with(
            mock_record_object.record_object_id
        )

    @patch.object(blob_utils, "blob_system_api")
    @patch.object(blob_utils, "local_id_system_api")
    @patch.object(blob_utils, "re")
    @patch.object(blob_utils, "reverse")
    @patch.object(blob_utils, "logger")
    def test_get_pid_for_blob_error_is_logged(
        self,
        mock_logger,
        mock_reverse,  # noqa, pylint: disable=unused-argument
        mock_regex_module,
        mock_local_id_system_api,  # noqa, pylint: disable=unused-argument
        mock_blob_system_api,
    ):
        """test_get_pid_for_blob_error_is_logged"""
        mock_regex_module.findall.return_value = [MagicMock()]
        mock_blob_system_api.get_pid_for_blob.side_effect = Exception(
            "mock_get_pid_for_blob_exception"
        )

        blob_utils.get_blob_download_regex("mock_xml_string")
        mock_logger.warning.assert_called()

    @patch.object(blob_utils, "blob_system_api")
    @patch.object(blob_utils, "local_id_system_api")
    @patch.object(blob_utils, "re")
    @patch.object(blob_utils, "reverse")
    def test_successful_execution_returns_blob_url_list(
        self,
        mock_reverse,  # noqa, pylint: disable=unused-argument
        mock_regex_module,
        mock_local_id_system_api,  # noqa, pylint: disable=unused-argument
        mock_blob_system_api,  # noqa, pylint: disable=unused-argument
    ):
        """test_successful_execution_returns_blob_url_list"""
        mock_document_list = [MagicMock()]
        mock_regex_module.findall.return_value = mock_document_list

        self.assertEqual(
            blob_utils.get_blob_download_regex("mock_xml_string"),
            mock_document_list,
        )



""" Local resolver API
"""
import logging

from core_linked_records_app.components.oai_record.access_control import (
    can_get_pid_for_data,
)
from core_linked_records_app.components.pid_path import api as pid_path_api
from core_linked_records_app.utils.dict import get_value_from_dot_notation
from core_main_app.access_control.decorators import access_control
from core_main_app.commons.exceptions import ApiError
from core_oaipmh_harvester_app.components.oai_record import (
    api as oai_record_data,
)

logger = logging.getLogger(__name__)


@access_control(can_get_pid_for_data)
def get_pid_for_data(oai_record_id, request):
    """Retrieve PID matching the document ID provided.

    Args:
        oai_record_id:
        request: HttpRequest

    Returns:
    """
    try:
        # Retrieve the document passed as input and extra the PID field.
        data = oai_record_data.get_by_id(oai_record_id, request.user)

        pid_path_object = pid_path_api.get_by_template(
            data.harvester_metadata_format.template, request.user
        )
        pid_path = pid_path_object.path

        # Return PID value from the document and the PID_PATH
        return get_value_from_dot_notation(data.get_dict_content(), pid_path)
    except Exception as exc:
        error_message = (
            "An unexpected error occurred while retrieving PID for OAI data"
        )

        logger.error("%s: %s", error_message, str(exc))
        raise ApiError(f"{error_message}.") from exc



""" Local record system implementation
"""
import json
import random
import string

from requests import Response
from rest_framework import status

from core_linked_records_app.components.local_id.models import LocalId
from core_linked_records_app.system.local_id import api as local_id_system_api
from core_linked_records_app.utils.providers import AbstractIdProvider
from core_main_app.commons import exceptions


class LocalIdProvider(AbstractIdProvider):
    """Local Id Provider"""

    messages = {
        "success": "Successful operation",
        "not_found": "Record not found",
        "already_exist": "Record already exists",
    }

    def __init__(self, provider_name):
        super().__init__(provider_name, None)

    @staticmethod
    def _generate_id(length_id=16):
        return "".join(
            random.choice(string.ascii_uppercase + string.digits)
            for _ in range(length_id)
        )

    def is_id_already_used(self, record):
        """is_id_already_used
        Args:
            record

        Returns:
        """
        return (
            json.loads(self.get(record).content)["message"]
            == self.messages["success"]
        )

    def get(self, record):
        """get

        Args:
            record:

        Returns:
        """

        response = Response()

        record_url = f"{self.provider_lookup_url}/{record}"

        try:
            local_id_system_api.get_by_name(record)
            response.status_code = status.HTTP_200_OK
            response._content = json.dumps(
                {
                    "message": self.messages["success"],
                    "record": record,
                    "url": record_url,
                }
            )
        except exceptions.DoesNotExist:
            response.status_code = status.HTTP_404_NOT_FOUND
            response._content = json.dumps(
                {
                    "message": self.messages["not_found"],
                    "record": record,
                    "url": record_url,
                }
            )

        return response

    def create(self, prefix, record=None):
        """create

        Args:
            prefix:
            record:

        Returns:
        """

        if record is None:
            # FIXME: duplicate code with core_main_registry_app
            # Create new record randomly
            record = self._generate_id()

            # While the record exists, retry creation of record
            while self.is_id_already_used(f"{prefix}/{record}"):
                record = self._generate_id()

        record = f"{prefix}/{record}"
        record_url = f"{self.provider_lookup_url}/{record}"

        response = Response()
        response_content = {
            "record": record,
            "url": record_url,
        }

        try:
            record_object = LocalId(record_name=record)

            local_id_system_api.insert(record_object)

            response.status_code = status.HTTP_201_CREATED
            response_content["message"] = self.messages["success"]
        except exceptions.NotUniqueError:
            response.status_code = status.HTTP_409_CONFLICT
            response_content["message"] = self.messages["already_exist"]

        response._content = json.dumps(response_content)
        return response

    def update(self, record):
        """update

        Args:
            record:

        Returns:
        """
        self.create(record)

        record_url = f"{self.provider_lookup_url}/{record}"

        response = Response()
        response._content = json.dumps(
            {
                "record": record,
                "message": self.messages["success"],
                "url": record_url,
            }
        )

        return response

    def delete(self, record):
        """delete

        Args:
            record:

        Returns:
        """
        response = Response()

        record_url = f"{self.provider_lookup_url}/{record}"

        try:
            record_object = local_id_system_api.get_by_name(record)
            local_id_system_api.delete(record_object)

            response.status_code = status.HTTP_200_OK
            response._content = json.dumps(
                {
                    "record": record,
                    "message": self.messages["success"],
                    "url": record_url,
                }
            )
        except exceptions.DoesNotExist:
            response.status_code = status.HTTP_404_NOT_FOUND
            response._content = json.dumps(
                {
                    "record": record,
                    "message": self.messages["not_found"],
                    "url": record_url,
                }
            )

        return response



""" Unit tests for PidPath admin site
"""
from unittest import TestCase
from unittest.mock import Mock

from core_linked_records_app.components.local_id.admin_site import (
    CustomLocalIdAdmin,
)


class TestCustomLocalIdAdminHasAddPermission(TestCase):
    """Test has_add_permission in CustomLocalIdAdmin view"""

    def test_cannot_add_item(self):
        """test_cannot_add_item"""
        custom_local_id_model_admin = CustomLocalIdAdmin(Mock(), Mock())
        self.assertFalse(
            custom_local_id_model_admin.has_add_permission(None, None)
        )


class TestCustomLocalIdAdminHasChangePermission(TestCase):
    """Test has_change_permission in CustomLocalIdAdmin view"""

    def test_cannot_edit_item(self):
        """test_cannot_edit_item"""
        custom_local_id_model_admin = CustomLocalIdAdmin(Mock(), Mock())
        self.assertFalse(
            custom_local_id_model_admin.has_change_permission(None, None)
        )



""" Access control methods for `core_linked_records.components.pid_settings.api`.
"""
from core_linked_records_app import settings
from core_linked_records_app.access_control import rights
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.access_control.utils import check_has_perm


def can_upsert_pid_settings(func, pid_settings_object, user):
    """Access control for the `upsert` function.

    Args:
        func:
        pid_settings_object:
        user:

    Returns:
    """
    if user.is_superuser:
        return func(pid_settings_object, user)

    raise AccessControlError("The user cannot upsert PidSettings objects.")


def can_get_pid_settings(func, user):
    """Access control for the `get` function.

    Args:
        func:
        user:

    Returns:
    """
    # If anonymous can access public document, then everyone can read PID settings.
    if not settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
        check_has_perm(user, rights.CAN_READ_PID_SETTINGS)

    return func(user)



""" Initialization function for PID Settings
"""
import logging

from core_linked_records_app import settings
from core_linked_records_app.components.pid_settings.models import PidSettings
from core_linked_records_app.system.pid_settings import (
    api as pid_settings_system_api,
)

logger = logging.getLogger(__name__)


def init():
    """Main initialization function"""
    try:
        if not PidSettings.get():
            pid_settings = PidSettings(auto_set_pid=settings.AUTO_SET_PID)
            pid_settings_system_api.upsert(pid_settings)
    except Exception as exc:  # pylint: disable=broad-except
        logger.error("Impossible to initialize PidSettings: %s", str(exc))



""" Unit tests for `core_linked_records.components.data.access_control`.
"""
from unittest import TestCase
from unittest.mock import MagicMock, patch

from core_linked_records_app.access_control import rights
from core_linked_records_app.components.pid_settings import (
    access_control as pid_settings_acl,
)
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestCanUpsertPidSetting(TestCase):
    """Unit tests for `can_upsert_pid_settings` function."""

    def setUp(self) -> None:
        """setUp"""
        self.user = create_mock_user("1")
        self.mock_kwargs = {
            "func": MagicMock(),
            "pid_settings_object": MagicMock(),
            "user": self.user,
        }

    def test_superuser_returns_func(self):
        """test_superuser_returns_func"""
        self.user.is_superuser = True

        pid_settings_acl.can_upsert_pid_settings(**self.mock_kwargs)
        self.mock_kwargs["func"].assert_called_with(
            self.mock_kwargs["pid_settings_object"], self.mock_kwargs["user"]
        )

    def test_not_superuser_raises_acl_error(self):
        """test_not_superuser_raises_acl_error"""
        with self.assertRaises(AccessControlError):
            pid_settings_acl.can_upsert_pid_settings(**self.mock_kwargs)


class TestCanGetPidSetting(TestCase):
    """Unit tests for `can_get_pid_settings` function."""

    def setUp(self) -> None:
        self.mock_kwargs = {
            "func": MagicMock(),
            "user": create_mock_user("1"),
        }

    @patch.object(pid_settings_acl, "check_has_perm")
    @patch.object(pid_settings_acl, "settings")
    def test_check_has_perm_not_called_when_public(
        self, mock_settings, mock_check_has_perm
    ):
        """test_check_has_perm_not_called_when_public"""
        mock_settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT = True
        pid_settings_acl.can_get_pid_settings(**self.mock_kwargs)
        mock_check_has_perm.assert_not_called()

    @patch.object(pid_settings_acl, "check_has_perm")
    @patch.object(pid_settings_acl, "settings")
    def test_check_has_perm_called_when_private(
        self, mock_settings, mock_check_has_perm
    ):
        """test_check_has_perm_called_when_private"""
        mock_settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT = False
        pid_settings_acl.can_get_pid_settings(**self.mock_kwargs)
        mock_check_has_perm.assert_called_with(
            self.mock_kwargs["user"], rights.CAN_READ_PID_SETTINGS
        )

    @patch.object(pid_settings_acl, "check_has_perm")
    @patch.object(pid_settings_acl, "settings")
    def test_check_has_perm_fail_raises_acl_error(
        self, mock_settings, mock_check_has_perm
    ):
        """test_check_has_perm_fail_raises_acl_error"""
        mock_settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT = False
        mock_check_has_perm.side_effect = AccessControlError(
            "test_check_has_perm_fail_raises_acl_error"
        )

        with self.assertRaises(AccessControlError):
            pid_settings_acl.can_get_pid_settings(**self.mock_kwargs)

    @patch.object(pid_settings_acl, "check_has_perm")
    @patch.object(pid_settings_acl, "settings")
    def test_successful_execution_returns_func_with_args(
        self,
        mock_settings,
        mock_check_has_perm,  # noqa, pylint: disable=unused-argument
    ):
        """test_successful_execution_returns_func_with_args"""
        mock_settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT = False
        self.assertEqual(
            pid_settings_acl.can_get_pid_settings(**self.mock_kwargs),
            self.mock_kwargs["func"](self.mock_kwargs["user"]),
        )



""" Unit tests for core_linked_records_app.components.data.api
"""
from unittest import TestCase
from unittest.mock import patch, Mock

from django.http import HttpRequest

from core_linked_records_app.components.data import (
    api as pid_data_api,
    access_control as pid_data_acl,
)
from core_linked_records_app.components.pid_path import api as pid_path_api
from core_main_app.commons import exceptions
from core_main_app.components.data import api as main_data_api
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from tests import mocks


class TestGetDataByPid(TestCase):
    """Test Get Data By Pid"""

    def setUp(self) -> None:
        mock_pid = "mock_pid"
        self.mock_user = create_mock_user("1")
        mock_request = Mock(spec=HttpRequest)
        mock_request.user = self.mock_user

        self.mock_kwargs = {"pid": mock_pid, "request": mock_request}

    @patch.object(pid_path_api, "get_all")
    def test_pid_path_get_all_failure_raises_api_error(self, mock_get_all):
        """test_pid_path_get_all_failure_raises_api_error"""

        mock_get_all.side_effect = Exception("mock_get_all_exception")

        with self.assertRaises(exceptions.ApiError):
            pid_data_api.get_data_by_pid(**self.mock_kwargs)

    @patch.object(main_data_api, "execute_json_query")
    @patch.object(pid_path_api, "get_all")
    def test_execute_query_failure_raises_api_error(
        self, mock_get_all, mock_execute_json_query
    ):
        """test_execute_query_failure_raises_api_error"""

        mock_get_all.return_value = []
        mock_execute_json_query.side_effect = Exception(
            "mock_execute_json_query_exception"
        )

        with self.assertRaises(exceptions.ApiError):
            pid_data_api.get_data_by_pid(**self.mock_kwargs)

    @patch.object(main_data_api, "execute_json_query")
    @patch.object(pid_path_api, "get_all")
    def test_no_result_raise_does_not_exist_error(
        self, mock_get_all, mock_execute_json_query
    ):
        """test_no_result_raise_does_not_exist_error"""

        mock_get_all.return_value = []
        mock_execute_json_query.return_value = []

        with self.assertRaises(exceptions.DoesNotExist):
            pid_data_api.get_data_by_pid(**self.mock_kwargs)

    @patch.object(main_data_api, "execute_json_query")
    @patch.object(pid_path_api, "get_all")
    def test_several_results_raise_api_error(
        self, mock_get_all, mock_execute_json_query
    ):
        """test_several_results_raise_api_error"""

        mock_get_all.return_value = []
        mock_execute_json_query.return_value = ["item_a", "item_b"]

        with self.assertRaises(exceptions.ApiError):
            pid_data_api.get_data_by_pid(**self.mock_kwargs)

    @patch.object(main_data_api, "execute_json_query")
    @patch.object(pid_path_api, "get_all")
    def test_single_result_is_returned(
        self, mock_get_all, mock_execute_json_query
    ):
        """test_single_result_is_returned"""

        mock_get_all.return_value = []
        expected_result = mocks.MockData()
        expected_result.user_id = self.mock_user.id
        mock_execute_json_query.return_value = [expected_result]

        result = pid_data_api.get_data_by_pid(**self.mock_kwargs)
        self.assertEqual(result, expected_result)


class TestGetPidForData(TestCase):
    """Test Get Pid For Data"""

    def setUp(self) -> None:
        mock_data_id = mocks.MockData().pk
        mock_user = create_mock_user("1")
        mock_request = Mock(spec=HttpRequest)
        mock_request.user = mock_user

        self.mock_kwargs = {"data_id": mock_data_id, "request": mock_request}

        self.mock_global_data = mocks.MockData()

    @patch.object(main_data_api, "get_by_id")
    @patch.object(pid_data_acl, "Data")
    @patch.object(pid_data_acl, "check_can_read_document")
    def test_get_by_id_failure_raises_api_error(
        self,
        mock_check_can_read_document,
        mock_data,  # noqa, pylint: disable=unused-argument
        mock_get_by_id,
    ):
        """test_get_by_id_failure_raises_api_error"""

        mock_check_can_read_document.return_value = True
        mock_get_by_id.side_effect = Exception("mock_get_by_id_exception")

        with self.assertRaises(exceptions.ApiError):
            pid_data_api.get_pid_for_data(**self.mock_kwargs)

    @patch.object(pid_path_api, "get_by_template")
    @patch.object(main_data_api, "get_by_id")
    @patch.object(pid_data_acl, "Data")
    @patch.object(pid_data_acl, "check_can_read_document")
    def test_get_by_template_failure_raises_api_error(
        self,
        mock_check_can_read_document,
        mock_data,  # noqa, pylint: disable=unused-argument
        mock_get_by_id,
        mock_get_by_template,
    ):
        """test_get_by_template_failure_raises_api_error"""
        mock_check_can_read_document.return_value = True
        mock_get_by_id.return_value = self.mock_global_data
        mock_get_by_template.side_effect = Exception(
            "mock_get_by_template_exception"
        )

        with self.assertRaises(exceptions.ApiError):
            pid_data_api.get_pid_for_data(**self.mock_kwargs)

    @patch.object(pid_data_api, "is_dot_notation_in_dictionary")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(main_data_api, "get_by_id")
    @patch.object(pid_data_acl, "Data")
    @patch.object(pid_data_acl, "check_can_read_document")
    def test_is_key_list_in_dictionary_failure_raises_api_error(
        self,
        mock_check_can_read_document,
        mock_data,  # noqa, pylint: disable=unused-argument
        mock_get_by_id,
        mock_get_by_template,
        mock_is_dot_notation_in_dictionary,
    ):
        """test_is_key_list_in_dictionary_failure_raises_api_error"""
        mock_check_can_read_document.return_value = True
        mock_get_by_id.return_value = self.mock_global_data
        mock_get_by_template.return_value = mocks.MockPidPath()
        mock_is_dot_notation_in_dictionary.side_effect = Exception(
            "mock_is_dot_notation_in_dictionary_exception"
        )

        with self.assertRaises(exceptions.ApiError):
            pid_data_api.get_pid_for_data(**self.mock_kwargs)

    @patch.object(pid_data_api, "is_dot_notation_in_dictionary")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(main_data_api, "get_by_id")
    @patch.object(pid_data_acl, "Data")
    @patch.object(pid_data_acl, "check_can_read_document")
    def test_is_dot_notation_in_dictionary_failure_raises_api_error(
        self,
        mock_check_can_read_document,
        mock_data,  # noqa, pylint: disable=unused-argument
        mock_get_by_id,
        mock_get_by_template,
        mock_is_dot_notation_in_dictionary,
    ):
        """test_is_dot_notation_in_dictionary_failure_raises_api_error"""
        mock_check_can_read_document.return_value = True
        mock_get_by_id.return_value = self.mock_global_data
        mock_get_by_template.return_value = mocks.MockPidPath()
        mock_is_dot_notation_in_dictionary.side_effect = Exception(
            "mock_is_dot_notation_in_dictionary_exception"
        )

        with self.assertRaises(exceptions.ApiError):
            pid_data_api.get_pid_for_data(**self.mock_kwargs)

    @patch.object(pid_data_api, "is_dot_notation_in_dictionary")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(main_data_api, "get_by_id")
    @patch.object(pid_data_acl, "Data")
    @patch.object(pid_data_acl, "check_can_read_document")
    def test_pid_path_not_in_document_returns_none(
        self,
        mock_check_can_read_document,
        mock_data,  # noqa, pylint: disable=unused-argument
        mock_get_by_id,
        mock_get_by_template,
        mock_is_dot_notation_in_dictionary,
    ):
        """test_pid_path_not_in_document_returns_none"""
        mock_check_can_read_document.return_value = True
        mock_get_by_id.return_value = self.mock_global_data
        mock_get_by_template.return_value = mocks.MockPidPath()
        mock_is_dot_notation_in_dictionary.return_value = False

        self.assertIsNone(pid_data_api.get_pid_for_data(**self.mock_kwargs))

    @patch.object(pid_data_api, "is_dot_notation_in_dictionary")
    @patch.object(pid_data_api, "get_value_from_dot_notation")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(main_data_api, "get_by_id")
    @patch.object(pid_data_acl, "Data")
    @patch.object(pid_data_acl, "check_can_read_document")
    def test_get_value_from_dot_notation_failure_raises_api_error(
        self,
        mock_check_can_read_document,
        mock_data,  # noqa, pylint: disable=unused-argument
        mock_get_by_id,
        mock_get_by_template,
        mock_get_value_from_dot_notation,
        mock_is_dot_notation_in_dictionary,
    ):
        """test_get_value_from_dot_notation_failure_raises_api_error"""
        mock_check_can_read_document.return_value = True
        mock_get_by_id.return_value = self.mock_global_data
        mock_get_by_template.return_value = mocks.MockPidPath()
        mock_get_value_from_dot_notation.return_value = True
        mock_is_dot_notation_in_dictionary.side_effect = Exception(
            "mock_get_dict_value_from_key_list_exception"
        )

        with self.assertRaises(exceptions.ApiError):
            pid_data_api.get_pid_for_data(**self.mock_kwargs)

    @patch.object(pid_data_api, "is_valid_pid_value")
    @patch.object(pid_data_api, "is_dot_notation_in_dictionary")
    @patch.object(pid_data_api, "get_value_from_dot_notation")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(main_data_api, "get_by_id")
    @patch.object(pid_data_acl, "Data")
    @patch.object(pid_data_acl, "check_can_read_document")
    def test_invalid_pid_raises_api_error(
        self,
        mock_check_can_read_document,
        mock_data,  # noqa, pylint: disable=unused-argument
        mock_get_by_id,
        mock_get_by_template,
        mock_get_value_from_dot_notation,
        mock_is_dot_notation_in_dictionary,
        mock_is_valid_pid_value,
    ):
        """test_invalid_pid_raises_api_error"""

        mock_check_can_read_document.return_value = True
        mock_get_by_id.return_value = self.mock_global_data
        mock_get_by_template.return_value = mocks.MockPidPath()
        mock_is_dot_notation_in_dictionary.return_value = True
        mock_get_value_from_dot_notation.return_value = "mock_pid"
        mock_is_valid_pid_value.return_value = False

        with self.assertRaises(exceptions.ApiError):
            pid_data_api.get_pid_for_data(**self.mock_kwargs)

    @patch.object(pid_data_api, "is_valid_pid_value")
    @patch.object(pid_data_api, "is_dot_notation_in_dictionary")
    @patch.object(pid_data_api, "get_value_from_dot_notation")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(main_data_api, "get_by_id")
    @patch.object(pid_data_acl, "Data")
    @patch.object(pid_data_acl, "check_can_read_document")
    def test_returns_get_value_from_dot_notation_output(
        self,
        mock_check_can_read_document,
        mock_data,  # noqa, pylint: disable=unused-argument
        mock_get_by_id,
        mock_get_by_template,
        mock_get_value_from_dot_notation,
        mock_is_dot_notation_in_dictionary,
        mock_is_valid_pid_value,
    ):
        """test_returns_get_value_from_dot_notation_output"""

        mock_check_can_read_document.return_value = True
        expected_result = "mock_pid"
        mock_get_by_id.return_value = self.mock_global_data
        mock_get_by_template.return_value = mocks.MockPidPath()
        mock_is_dot_notation_in_dictionary.return_value = True
        mock_get_value_from_dot_notation.return_value = expected_result
        mock_is_valid_pid_value.return_value = True

        result = pid_data_api.get_pid_for_data(**self.mock_kwargs)
        self.assertEqual(result, expected_result)



""" Serializers for calls related to `PidSettings` model.
"""
from rest_framework import serializers

from core_linked_records_app.components.pid_settings import (
    api as pid_settings_api,
)
from core_linked_records_app.components.pid_settings.models import (
    PidSettings,
)


class PidSettingsSerializer(serializers.ModelSerializer):
    """Pid Settings Serializer"""

    class Meta:
        """Meta"""

        model = PidSettings
        exclude = ["id"]

    def update(self, instance: PidSettings, validated_data):
        """Update a PidSetting instance

        Args:
            instance:
            validated_data:

        Returns:
            Updated PidSettings instance
        """
        instance.auto_set_pid = validated_data["auto_set_pid"]
        return pid_settings_api.upsert(instance, self.context["request"].user)



""" Unit test for REST query views
"""
from unittest import TestCase
from unittest.mock import patch

from core_linked_records_app.rest.query import views as query_views
from tests import mocks


class TestRetrieveQueryPidListViewPost(TestCase):
    """Unit tests for post function of RetrieveQueryPidListView class"""

    def setUp(self) -> None:
        """setUp"""
        self.test_view = query_views.RetrieveQueryPidListView()
        self.mock_request = mocks.MockRequest()

    @patch.object(query_views, "execute_local_pid_query")
    def test_execute_local_pid_query_error_returns_500(
        self, mock_execute_local_pid_query
    ):
        """test_execute_local_pid_query_error_returns_500"""
        mock_execute_local_pid_query.side_effect = Exception(
            "mock_execute_local_pid_query_exception"
        )

        response = self.test_view.post(self.mock_request)
        self.assertEqual(response.status_code, 500)

    @patch.object(query_views, "execute_local_pid_query")
    def test_success_returns_200(self, mock_execute_local_pid_query):
        """test_success_returns_200"""
        mock_execute_local_pid_query.return_value = [
            "mock_data_pid" for _ in range(5)
        ]

        response = self.test_view.post(self.mock_request)
        self.assertEqual(response.status_code, 200)

    @patch.object(query_views, "execute_local_pid_query")
    def test_success_returns_execute_local_pid_query(
        self, mock_execute_local_pid_query
    ):
        """test_success_returns_execute_local_pid_query"""
        expected_result = ["mock_data_pid" for _ in range(5)]
        mock_execute_local_pid_query.return_value = expected_result

        response = self.test_view.post(self.mock_request)
        self.assertEqual(response.data, expected_result)



""" Unit tests for core_linked_records_app.rest.pid.views
"""
import json
from unittest import TestCase
from unittest.mock import patch

from core_explore_common_app.components.query import api as query_api
from core_explore_common_app.utils.oaipmh import oaipmh as oaipmh_utils
from core_explore_common_app.utils.query import query as query_utils
from core_federated_search_app.components.instance import (
    api as instance_api,
)
from core_linked_records_app.components.blob import api as blob_api
from core_linked_records_app.components.data import api as data_api
from core_linked_records_app.components.oai_record import (
    api as oai_record_api,
)
from core_linked_records_app.rest.pid import views as pid_views
from tests import mocks


class TestRetrieveDataPidGet(TestCase):
    """Test Retrieve Data Pid Get"""

    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest()
        self.mock_request.GET = {}

    @patch.object(data_api, "get_pid_for_data")
    def test_data_api_get_pid_for_data_fails_returns_500(
        self, mock_get_pid_for_data
    ):
        """test_data_api_get_pid_for_data_fails_returns_500"""

        self.mock_request.GET["data_id"] = "mock_data_id"
        mock_get_pid_for_data.side_effect = Exception(
            "mock_get_pid_for_data_exception"
        )

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(data_api, "get_pid_for_data")
    def test_data_api_success_returns_200(self, mock_get_pid_for_data):
        """test_data_api_success_returns_200"""

        self.mock_request.GET["data_id"] = "mock_data_id"
        mock_get_pid_for_data.return_value = "mock_data_id"

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEqual(response.status_code, 200)

    @patch.object(oai_record_api, "get_pid_for_data")
    def test_oai_record_api_get_pid_for_data_fails_returns_500(
        self, mock_get_pid_for_data
    ):
        """test_oai_record_api_get_pid_for_data_fails_returns_500"""
        self.mock_request.GET["oai_data_id"] = "mock_data_id"
        mock_get_pid_for_data.side_effect = Exception(
            "mock_get_pid_for_data_exception"
        )

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(oai_record_api, "get_pid_for_data")
    def test_oai_record_api_success_returns_200(self, mock_get_pid_for_data):
        """test_oai_record_api_success_returns_200"""
        self.mock_request.GET["oai_data_id"] = "mock_data_id"
        mock_get_pid_for_data.return_value = "mock_data_id"

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEqual(response.status_code, 200)

    @patch.object(instance_api, "get_by_name")
    def test_instance_api_get_by_name_fails_returns_500(
        self, mock_get_by_name
    ):
        """test_instance_api_get_by_name_fails_returns_500"""

        self.mock_request.GET["fede_data_id"] = "mock_data_id"
        self.mock_request.GET["fede_origin"] = "mock_origin&param=mock_param"
        mock_get_by_name.side_effect = Exception("mock_get_by_name_exception")

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(pid_views, "oauth2_get_request")
    @patch.object(instance_api, "get_by_name")
    def test_oauth2_get_request_fails_returns_500(
        self, mock_get_by_name, mock_oauth2_get_request
    ):
        """test_oauth2_get_request_fails_returns_500"""

        self.mock_request.GET["fede_data_id"] = "mock_data_id"
        self.mock_request.GET["fede_origin"] = "mock_origin&param=mock_param"
        mock_get_by_name.return_value = mocks.MockInstance()
        mock_oauth2_get_request.side_effect = Exception(
            "mock_oauth2_get_request_exception"
        )

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(pid_views, "oauth2_get_request")
    @patch.object(instance_api, "get_by_name")
    def test_fede_success_returns_200(
        self, mock_get_by_name, mock_oauth2_get_request
    ):
        """test_fede_success_returns_200"""

        self.mock_request.GET["fede_data_id"] = "mock_data_id"
        self.mock_request.GET["fede_origin"] = "mock_origin&param=mock_param"
        mock_get_by_name.return_value = mocks.MockInstance()
        mock_oauth2_get_request.return_value = mocks.MockResponse(
            text=json.dumps({"key": "mock_response_text"})
        )

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEqual(response.status_code, 200)

    def test_incorrect_params_returns_400(self):
        """test_incorrect_params_returns_400"""

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEqual(response.status_code, 400)


class TestRetrieveBlobPidGet(TestCase):
    """Test Retrieve Blob Pid Get"""

    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest()
        self.mock_request.GET = {}

    @patch.object(blob_api, "get_pid_for_blob")
    def test_get_pid_for_blob_fails_returns_500(self, mock_get_pid_for_blob):
        """test_get_pid_for_blob_fails_returns_500"""

        self.mock_request.GET["blob_id"] = "mock_blob_id"
        mock_get_pid_for_blob.side_effect = Exception(
            "mock_get_pid_for_blob_exception"
        )

        test_view = pid_views.RetrieveBlobPIDView()
        response = test_view.get(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(blob_api, "get_pid_for_blob")
    def test_success_returns_200(self, mock_get_pid_for_blob):
        """test_success_returns_200"""

        self.mock_request.GET["blob_id"] = "mock_blob_id"
        mock_get_pid_for_blob.return_value = mocks.MockLocalId()

        test_view = pid_views.RetrieveBlobPIDView()
        response = test_view.get(self.mock_request)

        self.assertEqual(response.status_code, 200)

    def test_incorrect_params_returns_400(self):
        """test_incorrect_params_returns_400"""

        test_view = pid_views.RetrieveBlobPIDView()
        response = test_view.get(self.mock_request)

        self.assertEqual(response.status_code, 400)


class TestRetrieveListPidPost(TestCase):
    """Test Retrieve List Pid Post"""

    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest(
            data={"query_id": "mock_query_id"}
        )

    @patch.object(query_api, "get_by_id")
    def test_get_by_id_fails_returns_500(self, mock_get_by_id):
        """test_get_by_id_fails_returns_500"""

        mock_get_by_id.side_effect = Exception("mock_get_by_id_exception")

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(query_api, "get_by_id")
    def test_data_source_invalid_returns_500(self, mock_get_by_id):
        """test_data_source_invalid_returns_500"""
        mock_get_by_id.return_value = mocks.MockQuery(data_sources=[])

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(query_api, "get_by_id")
    def test_unknown_datasource_authtype_returns_400(self, mock_get_by_id):
        """test_unknown_auth_returns_500"""

        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                dict(
                    query_options={},
                    order_by_field="",
                    capabilities={},
                    authentication=dict(
                        auth_type="invalid_auth_type",
                    ),
                )
            ]
        )

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 400)

    @patch.object(query_api, "get_by_id")
    def test_invalid_datasource_auth_type_returns_500(self, mock_get_by_id):
        """test_unknown_auth_returns_500"""

        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                dict(
                    query_options={},
                    order_by_field="",
                    capabilities={},
                    authentication={},
                )
            ]
        )

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(oaipmh_utils, "is_oai_data_source")
    @patch.object(query_utils, "is_local_data_source")
    @patch.object(query_api, "get_by_id")
    def test_invalid_session_datasource_returns_500(
        self,
        mock_get_by_id,
        mock_is_local_data_source,
        mock_is_oai_data_source,
    ):
        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                dict(
                    query_options={},
                    order_by_field="",
                    capabilities={},
                    authentication=dict(
                        auth_type="session",
                    ),
                )
            ]
        )
        mock_is_local_data_source.return_value = False
        mock_is_oai_data_source.return_value = False

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(pid_views, "execute_local_pid_query")
    @patch.object(oaipmh_utils, "is_oai_data_source")
    @patch.object(query_utils, "is_local_data_source")
    @patch.object(query_api, "get_by_id")
    def test_local_datasource_returns_200(
        self,
        mock_get_by_id,
        mock_is_local_data_source,
        mock_is_oai_data_source,
        mock_execute_local_pid_query,
    ):
        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                dict(
                    query_options={},
                    order_by_field="",
                    capabilities={},
                    authentication=dict(
                        auth_type="session",
                    ),
                )
            ]
        )
        mock_is_local_data_source.return_value = True
        mock_is_oai_data_source.return_value = False
        mock_execute_local_pid_query.return_value = []

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 200)

    @patch.object(pid_views, "execute_local_pid_query")
    @patch.object(oaipmh_utils, "is_oai_data_source")
    @patch.object(query_utils, "is_local_data_source")
    @patch.object(query_api, "get_by_id")
    def test_local_datasource_returns_local_pid_query(
        self,
        mock_get_by_id,
        mock_is_local_data_source,
        mock_is_oai_data_source,
        mock_execute_local_pid_query,
    ):
        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                dict(
                    query_options={},
                    order_by_field="",
                    capabilities={},
                    authentication=dict(
                        auth_type="session",
                    ),
                )
            ]
        )
        mock_is_local_data_source.return_value = True
        mock_is_oai_data_source.return_value = False

        expected_result = ["mock_pid" for _ in range(5)]
        mock_execute_local_pid_query.return_value = expected_result

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)
        result = response.data["pids"]

        self.assertEqual(result, expected_result)

    @patch.object(pid_views, "execute_oaipmh_pid_query")
    @patch.object(oaipmh_utils, "is_oai_data_source")
    @patch.object(query_utils, "is_local_data_source")
    @patch.object(query_api, "get_by_id")
    def test_oaipmh_datasource_returns_200(
        self,
        mock_get_by_id,
        mock_is_local_data_source,
        mock_is_oai_data_source,
        mock_execute_oaipmh_pid_query,
    ):
        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                dict(
                    query_options={},
                    order_by_field="",
                    capabilities={},
                    authentication=dict(
                        auth_type="session",
                    ),
                )
            ]
        )
        mock_is_local_data_source.return_value = False
        mock_is_oai_data_source.return_value = True
        mock_execute_oaipmh_pid_query.return_value = []

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 200)

    @patch.object(pid_views, "execute_oaipmh_pid_query")
    @patch.object(oaipmh_utils, "is_oai_data_source")
    @patch.object(query_utils, "is_local_data_source")
    @patch.object(query_api, "get_by_id")
    def test_oaipmh_datasource_returns_oaipmh_pid_query(
        self,
        mock_get_by_id,
        mock_is_local_data_source,
        mock_is_oai_data_source,
        mock_execute_oaipmh_pid_query,
    ):
        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                dict(
                    query_options={},
                    order_by_field="",
                    capabilities={},
                    authentication=dict(
                        auth_type="session",
                    ),
                )
            ]
        )
        mock_is_local_data_source.return_value = False
        mock_is_oai_data_source.return_value = True
        expected_result = ["mock_pid" for _ in range(5)]
        mock_execute_oaipmh_pid_query.return_value = expected_result

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)
        result = response.data["pids"]

        self.assertEqual(result, expected_result)

    # FIXME need federation fixing first
    @patch.object(pid_views, "oauth2_post_request")
    @patch.object(query_api, "get_by_id")
    def test_oauth2_post_request_fails_returns_500(
        self, mock_get_by_id, mock_oauth2_post_request
    ):
        """test_oauth2_post_request_fails_returns_500"""

        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                dict(
                    query_options={},
                    order_by_field="",
                    capabilities={"query_pid": "mock_url_pid"},
                    authentication=dict(
                        auth_type="oauth2",
                        params={"access_token": "mock_access_token"},
                    ),
                )
            ]
        )
        mock_oauth2_post_request.side_effect = Exception(
            "mock_oauth2_post_request_exception"
        )

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(pid_views, "oauth2_post_request")
    @patch.object(query_api, "get_by_id")
    def test_status_200_returns_200(
        self, mock_get_by_id, mock_oauth2_post_request
    ):
        """test_status_200_returns_200"""

        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                dict(
                    query_options={},
                    order_by_field="",
                    capabilities={"query_pid": "mock_url_pid"},
                    authentication=dict(
                        auth_type="oauth2",
                        params={"access_token": "mock_access_token"},
                    ),
                )
            ]
        )
        mock_oauth2_post_request.return_value = mocks.MockResponse(
            json_data={"pids": []}
        )

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 200)

    @patch.object(pid_views, "oauth2_post_request")
    @patch.object(query_api, "get_by_id")
    def test_status_400_returns_400(
        self, mock_get_by_id, mock_oauth2_post_request
    ):
        """test_status_400_returns_400"""

        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                dict(
                    query_options={},
                    order_by_field="",
                    capabilities={"query_pid": "mock_url_pid"},
                    authentication=dict(
                        auth_type="oauth2",
                        params={"access_token": "mock_access_token"},
                    ),
                )
            ],
        )
        mock_oauth2_post_request.return_value = mocks.MockResponse(
            status_code=400
        )

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 400)



""" Access control for `core_linked_records_app.components.blob.api`.
"""
from core_main_app.access_control.api import check_can_read_document
from core_main_app.components.blob.models import Blob


def can_get_blob_by_pid(func, pid, user):
    """Check that a user can retrieve a `Blob` from its PID.

    Args:
        func:
        pid:
        user:

    Raises:
        AccessControlError: The user does not have the proper permissions.

    Returns:
        Blob: Blob object to be returned by the function.
    """
    blob = func(pid, user)

    if not user.is_superuser:
        check_can_read_document(blob, user)
    return blob


def can_get_pid_for_blob(func, blob_id, user):
    """Check that a use can read a `Blob` from its id and retrieve the attached PID
    value.

    Args:
        func:
        blob_id:
        user:

    Raises:
        AccessControlError: The user does not have the proper permissions.

    Returns:
        any: Runs the function specified in parameters.
    """
    if not user.is_superuser:
        check_can_read_document(Blob.get_by_id(blob_id), user)
    return func(blob_id, user)


def can_set_pid_for_blob(func, blob_id, blob_pid, user):
    """Check that a user can assign a PID value a Blob object given its id.

    Args:
        func:
        blob_id:
        blob_pid:
        user:

    Raises:
        AccessControlError: The user does not have the proper permissions.

    Returns:
        any: Runs the function specified in parameters.
    """
    if not user.is_superuser:
        check_can_read_document(Blob.get_by_id(blob_id), user)
    return func(blob_id, blob_pid, user)
